<?php

namespace Lumpik;

use FML\ManiaCode;
use FML\ManiaLink;
use FML\Controls\Control;
use FML\Controls\Frame;
use FML\Controls\Label;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Labels\Label_Button;
use FML\Controls\Quad;
use FML\Controls\Quads\Quad_Icons64x64_1;
use FML\Controls\Quads\Quad_BgRaceScore2;
use ManiaControl\Admin\AuthenticationManager;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Callbacks\Structures\ArmorEmptyStructure;
use ManiaControl\Callbacks\Structures\NearMissStructure;
use ManiaControl\Callbacks\Structures\CaptureStructure;
use ManiaControl\Callbacks\Structures\PlayerHitStructure;
use ManiaControl\Commands\CommandListener;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Manialinks\IconManager;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Players;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;


// Include FML
require_once __DIR__ . '/../../libs/FML/autoload.php';

// DEBUG MODE
define('ManiaControl\DEV_MODE', false);



/**
 * Sound Events Manager (SEM) plugin.
 * 
 * For any bug/suggestion : https://forum.maniaplanet.com/viewtopic.php?f=479&t=30285
 * 
 * 
 * V1.00 : Initial version
 * V1.01 : Music player, RESET_ON_NEW_ROUND and URL parameters
 * V1.02 : MESSAGES and RANKING can be enabled/disabled by players with new icons. All icons are gathered into new menu. Default RESET_ON_NEW_ROUND set to true for Siege.
 * V1.03 : Players can enable/disable MUSIC. HIT sounds are differentiated for RAIL, ROCKET, NUCLEUS and ARROW. RANKING is aligned on the right. Description label added on SEM icons.
 * V1.04 : Store user parameters (SOUNDS, MESSAGES, RANKING, MUSIC enabled/disabled). Some sounds can be parametered to be played to no one, only the player who triggered the sound, team of this player or everybody. New SEM icon. More understandable icons. No more "Playing music" message when no link is set. Using dynamical constants for KILL_*_LINK. Display "RANKING" title. Keep SEM menu opened. Some bug fixes. NB : Can't handle HEADSHOT, due to lake of datas from ManiaControl.
 * V1.05 : Fix ManiaControl crash on "Play music". Fix bug "Stop music" not displayed. Fix bugs with MESSAGES_ENABLED and RANKING_ENABLED (didn't work). Don't open SEM menu when restarting ManiaControl.
 * V1.06 : New menu icon (thanks to swinn for the icon). Add new parameters : MUSIC_DELAY, BEGIN_MAP_DELAY, BEGIN_PLAYING_DELAY, TEAM_SCORES_DELAY and SEM_MESSAGES_DURATION. New parameters to set width and height of all icons. KILLSTREAK and RANKING fixed on all FFA modes like Royal (thanks to YamYam for the report). Preload all sounds when connecting or restarting plugins. Remove some PHP notices sent to ManiaControl. Fix bug "play/stop music" icon not up-to-date when playing a music at beginning of new map. Bug fix : your icons won't change when other players change their configuration.
 * V1.07 : Export all "*_LINK" parameters into a config.xml. Renamed all other parameters.
 * V1.08 : Add SOUND ON DEMAND feature to allow admins to play their own sounds. Add config_default.xml and soundOnDemand_default.xml files to prevent deleting user config.
 * V1.09 : Add delays between each SOUND ON DEMAND in parameters. Players can mute SOUND ON DEMAND. "Start music" button is hidden if no link is set. Bug fix : messages displayed while no killstreak sound is played. 
 * V1.10 : Add <blockothersounds> and <duration> tags into XML files to avoid sounds/SOD to be stopped by other sounds. Add 2 parameters to hide "Preloading..." and "Playing Sound On Demand..." information messages. Add parameter to set volume of SOD. Removed all "_LINK" suffixes in config.xml. Add SEM version into information messages. 
 * 
 * @author Lumpik
 *
 */
 
 
 
 /*
	V1.11 - TO DO :
		++ TO DO : refaire le point sur l'utilité de MUSIC_BLOCKOTHERSOUNDS et MUSIC_DURATION
		++ TO DO : créer et gérer un fichier music.xml qui liste les musiques jouables, à la manière des SOD
			si des liens MUSIC sont présents, ajouter liens :
				"stop"
				"play all"		(stop + play chaque music dans l'ordre. si fin de liste, recommencer au début)
				"play random"	(stop + play random)
				music 1			(clic = "play in loop")
				music 2			(clic = "play in loop")
				music 3			(clic = "play in loop")
			
			--> utiliser "playlist" plutot que "allMusic" et "randomPlaylist" au lieu de "RandomMusic" ?
			
			dans load() : faire appel à buildMusicAll() et buildMusicRandom()
			dans buildMusicAll() : vider musicAll[] et alimenter un tableau musicAll[] avec toutes les musiques dans l'ordre (name, link, duration)
			dans buildMusicRandom() : vider musicRandom[] et alimenter un un tableau musicRandom[] avec toutes les musiques dans l'ordre (name, link, duration) et le shuffle()
			dans endOfAllMusic() : si $musicIsPlaying == false : return. Sinon, faire appel à playAllMusic()
			dans endOfRandomMusic() : si $musicIsPlaying == false : return. Sinon, faire appel à playRandomMusic()
			
			STOP :
				faire appel à playMusic() avec un lien vide et une duration presque nulle.
				$musicIsPlaying = false;
				faire appel à buildMusicAll();
				faire appel à buildMusicRandom();
			
			PLAY ALL : playAllMusic()
				display "Playing music $name"
				faire appel à playMusic() avec le 1er élément du tableau musicAll[]
				$musicIsPlaying = true;
				déplacer le 1er élément à la fin du tableau
				faire un appel différé à endOfAllMusic() au bout de $duration
			
			PLAY RANDOM : playRandomMusic()
				display "Playing randomly music $name"
				faire appel à playMusic() avec le 1er élément du tableau musicRandom[]
				$musicIsPlaying = true;
				déplacer le 1er élément à la fin du tableau
				faire un appel différé à endOfRandomMusic() au bout de $duration
				
			--> ATTENTION aux appels différés qui peuvent appeler une fonction alors que l'on ne veut pas : double-clic != timer
			--> faire un shuffle() à chaque STOP ou à chaque PLAY RANDOM ? 2eme solution semble la plus logique
		
		++ TO DO - gestion des priorités de lecture des SOUND et SOD : Créer attributs <newsound> et <newsod> dans chaque <sound> des 2 xml et qui prennent [cancel/play/force] en valeur. Supprimer <blockothersounds>
		
			Définitions :
				cancel : cancel the requested SOUND/SOD until the currently playing SOUND/SOD is over
				play : play the requested SOUND/SOD while a SOUND/SOD is already playing, so both SOUND/SOD will be played simultaneously
				force : stop the currently playing SOUND/SOD and play the requested SOUND/SOD
		
			Description des attributs :
				'While SOUND is playing : [cancel/play/force] new SOUND'
					What to do if a SOUND would be played while a SOUND is already playing ?
				'While SOUND is playing : [cancel/play/force] new SOD'
					What to do if a SOUND would be played while a SOD is already playing ?
				'While SOD is playing : [cancel/play/force] new SOUND'
					What to do if a SOD would be played while a SOUND is already playing ?
				' While SOD is playing : [cancel/play/force] new SOD'
					What to do if a SOD would be played while a SOD is already playing ?
			
			--> possibilité de créer l'option "delay" qui jouera le SOUND/SOD à la fin de celui en cours (cf resetSoundsBlocked()) ?
				si "delay", ajouter les infos du son dans un tableau soundsToPlay[]
				dans resetSoundsBlocked(), si soundsToPlay[] contient des éléments, jouer le 1er son de la liste et le supprimer du tableau
			
			How To :
			pour jouer le son demandé, en fonction de ses paramètres :
				- <newsound>cancel</newsound>	: si 'type == 'sound' && $soundsBlocked == false : utiliser ML_SOUND_ID, soundsBlocked = true, appel différé à resetSoundsBlocked()
				- <newsod>cancel</newsod>		: si 'type == 'sod' && $sodBlocked == false : utiliser ML_SOUND_ON_DEMAND_ID, sodBlocked = true, appel différé à resetSodBlocked() --> OK
				- <newsound>play</newsound>		: si 'type == 'sound' && $soundsBlocked == false, hidemanialink(ML_SOUND_ID), utiliser "ML_SOUND_ID . time()" (ID unique)
				- <newsod>play</newsod>			: si 'type == 'sod'  && $sodBlocked == false: hidemanialink(ML_SOUND_ON_DEMAND_ID), utiliser "ML_SOUND_ON_DEMAND_ID . time()" (ID unique)
				- <newsound>force</newsound>	: si 'type == 'sound' && $soundsBlocked == false : utiliser ML_SOUND_ID
				- <newsod>force</newsod>		: si 'type == 'sod' && $sodBlocked == false : utiliser ML_SOUND_ON_DEMAND_ID
				
			
		++ TO DO : callback headshot doit être gérable avec les nouveaux paramètres envoyés ? N'envoyer le son+message qu'au joueur
		++ Trigger for FIRST_HIT, FIRST_DEATH. 1 by round.
		
		TO DO - BEGIN_ROUND and END_ROUND: to add ?
		TO DO - CALLBACKS : décommenter toutes les lignes en "v1.08" et tester allow/son pour callbacks ONNEARMISS, BEGINWARMUP, ENDWARMUP, ENDPLAYING, ENDMAP.
		TO DO : activer/désactiver par défaut les sons/ranking/... pour les joueurs : OSEF !
		
		dans le tchat : /lol, /hs, /hi, etc...  pourrait jouer des sons
		onCapture : si Royal et pas d'info dans le CaptureStructure : automatical capture.
	
	V2.00 ou plus tard :
		++ TO CHECK : SIEGE : si possible, jouer un son sur la capture de CP --> quel callback est envoyé ??
		++ MUSIC : 
			TO TEST - music : récupérer le nom (aléatoire) des fichiers d'un répertoire.
			music : Ajouter option random. Si pas random, garder le dernier fichier lu en mémoire et lire le suivant la prochaine fois. Boucler si fin de la liste dans tous les cas.
			music : pouvoir jouer la musique que pour certaines personnnes
			music : l'icone ne se met pas à jour
		
		CTF : les callbacks ne sont pas récupérées. Pas envoyées ou mal catchées ? Tester sur serveur MM² temporaire
		ajouter les callbacks pour les phases : possible dans une prochaine version de SB
		ajouter Red_Team_takes_the_lead.wav ? Tester GetRoundCustomPoints() from http://methods.xaseco.org/methodstmc.php
		son spécial pour un kill du porteur de ball
				
		IMPOSSIBLE - CALLBACKS V3 : ajouter CB_MP_PLAYERALLIESCHANGED (KO), CB_MP_PLAYERINFOCHANGED (KO Nadeo ? impossible d'apres Godo). Cf https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/CallbackManager.php
 */

 
 /*
 LIENS UTILES :
 http://www.maniacontrol.com/user/plugins/70
 http://www.maniacontrol.com/docs
 http://slig.info/ms/docs/2012-08-14/html/annotated.html
 
 SPEEDBALL GITHUB :
 https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Modes/ShootMania/SpeedBall.Script.txt
 
 UT sounds :
 http://unrealtournament.99.free.fr/utfiles/index.php?dir=sound%20ut/
 
 LIST OF KILL AWARDS (check Unreal Tournament) :
 http://en.wikipedia.org/wiki/Deathmatch
 
 PLAYING SOUNDS WITH FML (Fancy Mania Links) :
 https://forum.maniaplanet.com/viewtopic.php?f=465&t=26099
 https://github.com/steeffeen/FancyManiaLinks/blob/master/examples/audio.php
 https://github.com/steeffeen/FancyManiaLinks/blob/master/FML/Controls/Audio.php
 https://github.com/steeffeen/FancyManiaLinks/blob/master/FML/Types/Playable.php
 https://github.com/steeffeen/FancyManiaLinks/blob/master/FML/Controls/Control.php
 
 
 https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/LibXmlRpcCallbacks.php
 */

 


class SoundEventsManager implements CallbackListener, CommandListener, TimerListener, ManialinkPageAnswerListener, Plugin {
	
    /**
     * Constants
     */
    const ID			= 70;
    const VERSION		= 1.10;
    const NAME			= 'Sound Events Manager';
    const AUTHOR		= 'Lumpik';
	const DESCRIPTION	= 'Play sounds on triggers, such as kills, beginning of round, etc...';
	
	// ManiaLinks ID
	const ML_ICON_MENU_ID			= 'SEM.ManiaLink.Icon.Menu.Id';
	const ML_ICON_MUSIC_ID			= 'SEM.ManiaLink.Icon.Music.Id';
	const ML_ICON_PLAY_ID			= 'SEM.ManiaLink.Icon.Play.Id';
	const ML_ICON_SOUND_ID			= 'SEM.ManiaLink.Icon.Sound.Id';
	const ML_ICON_MESSAGE_ID		= 'SEM.ManiaLink.Icon.Message.Id';
	const ML_ICON_RANKING_ID		= 'SEM.ManiaLink.Icon.Ranking.Id';
	const ML_ICON_SOD_MENU_ID		= 'SEM.ManiaLink.Icon.SoundOnDemand.Menu.Id';
	const ML_ICON_SOD_ON_OFF_ID		= 'SEM.ManiaLink.Icon.SoundOnDemand.On.Off.Id';
	const ML_SOUND_ID				= 'SEM.ManiaLink.Sound.Id';
	const ML_MUSIC_ID				= 'SEM.ManiaLink.Music.Id';
	const ML_MESSAGE_ID				= 'SEM.ManiaLink.Message.Id';
	const ML_RANKING_ID				= 'SEM.ManiaLink.Ranking.Id';
	const ML_SOUND_ON_DEMAND_ID		= 'SEM.ManiaLink.SoundOnDemand.Id';
	
	// Icons for menu
	const SEM_ICON_MENU				= 'menu.png';
	const SEM_ICON_MENU_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MENU_MOVER		= 'menu_mover.png';
	const SEM_ICON_MENU_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling sounds
	// Icons from http://www.softicons.com/search?search=iconza+sound&x=0&y=0
	const SEM_ICON_SOUND_ON				= 'sound_on.png';
	const SEM_ICON_SOUND_ON_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOUND_OFF			= 'sound_off.png';
	const SEM_ICON_SOUND_OFF_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling messages
	// Icons from http://www.softicons.com/search?search=iconza+text&x=0&y=0
	const SEM_ICON_MESSAGE_ON				= 'message_on.png';
	const SEM_ICON_MESSAGE_ON_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MESSAGE_OFF				= 'message_off.png';
	const SEM_ICON_MESSAGE_OFF_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling ranking
	// Icons from http://www.softicons.com/search?search=iconza+star&x=0&y=0
	// Other idea : http://www.softicons.com/sport-icons/pretty-office-vii-icons-by-custom-icon-design/podium-icon
	const SEM_ICON_RANKING_ON				= 'ranking_on.png';
	const SEM_ICON_RANKING_ON_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_RANKING_OFF				= 'ranking_off.png';
	const SEM_ICON_RANKING_OFF_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling music
	// Icons from http://www.softicons.com/search?search=iconza+music&x=0&y=0
	const SEM_ICON_MUSIC_ON				= 'music_on.png';
	const SEM_ICON_MUSIC_ON_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_OFF			= 'music_off.png';
	const SEM_ICON_MUSIC_OFF_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for playing/stopping music
	// Icons PLAY from http://www.softicons.com/search?search=Media+Square+Play+Icon&x=0&y=0
	// Icons STOP from http://www.softicons.com/search?search=Media+Black+Stop+Icon&x=0&y=0
	const SEM_ICON_MUSIC_PLAY				= 'music_play.png';
	const SEM_ICON_MUSIC_PLAY_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_STOP				= 'music_stop.png';
	const SEM_ICON_MUSIC_STOP_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for Sound On Demand
	// Icon SOD from http://www.softicons.com/web-icons/blue-gradient-toolbar-icons-by-axialis-team/sound-plus-icon
	// Icon SOD_MOVER from http://www.softicons.com/web-icons/web-blue-buttons-by-axialis-team/sound-plus-icon
	const SEM_ICON_SOD				= 'sound_on_demand.png';
	const SEM_ICON_SOD_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOD_MOVER		= 'sound_on_demand_mover.png';
	const SEM_ICON_SOD_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling Sound On Demand
	// Icon SOD ON from http://www.softicons.com/web-icons/web-blue-buttons-by-axialis-team/music-2-icon
	// Icon SOD OFF from http://www.softicons.com/web-icons/web-grey-buttons-by-axialis-team/music-2-icon
	const SEM_ICON_SOD_ON				= 'sound_on_demand_on.png';
	const SEM_ICON_SOD_ON_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOD_OFF				= 'sound_on_demand_off.png';
	const SEM_ICON_SOD_OFF_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	///////////////
	// PARAMETERS
	
	// URL TO FORUM
	const URL	= '000. For any bug/suggestion/...';
	
	// SOUND ON DEMAND parameters
	const SEM_SOUND_ON_DEMAND_POS_X				= '001a. SOUND ON DEMAND : X position';
	const SEM_SOUND_ON_DEMAND_POS_Y				= '001b. SOUND ON DEMAND : Y position';
	const SEM_SOUND_ON_DEMAND_GLOBAL_DELAY		= '001c. SOD : delay in ms between 2 sounds';
	const SEM_SOUND_ON_DEMAND_USER_DELAY		= '001d. SOD : delay in ms between 2 sounds from a same user';
	const SEM_SOUND_ON_DEMAND_MESSAGE_ACTIVATED	= '001e. SOD : display message when playing SOD ?';
	
	// DEBUG + VOLUME parameters
	const DEBUG			= '002. DEBUG';
	const PRELOAD_MESSAGE_ACTIVATED	= '002b. Display "Prealoading" message when you connect';
	const VOLUME_SOUND				= '003a. VOLUME for triggered sounds';
	const VOLUME_SOUND_ON_DEMAND	= '003b. VOLUME for SOUND ON DEMAND';
	
	// ICON MENU parameters
	const SEM_ICON_MENU_POS_X	= '004a. ICON MENU : X position';
	const SEM_ICON_MENU_POS_Y	= '004b. ICON MENU : Y position';
	const SEM_ICON_MENU_WIDTH	= '004c. ICON MENU : width';
	const SEM_ICON_MENU_HEIGHT	= '004d. ICON MENU : height';
	
	// MUSIC parameters
	const MUSIC_ENABLED	= '005a. MUSIC enabled on new map ?';
	const MUSIC_VOLUME	= '005b. MUSIC VOLUME';
	public $MUSIC_LINK;
	const MUSIC_DELAY	= '005c. MUSIC delay in ms on new map';
	
	// MESSAGE
	const SEM_MESSAGES_ENABLED	= '006a. MESSAGES enabled ?';
	const SEM_MESSAGES_POS_X	= '006b. MESSAGES : X position';
	const SEM_MESSAGES_POS_Y	= '006c. MESSAGES : Y position';
	const SEM_MESSAGES_SCALE	= '006d. MESSAGES : scale';
	const SEM_MESSAGES_DURATION	= '006e. MESSAGES : duration';
	
	// RANKING
	const SEM_RANKING_ENABLED	= '007a. RANKING enabled ?';
	const SEM_RANKING_POS_X		= '007b. RANKING : X position';
	const SEM_RANKING_POS_Y		= '007c. RANKING : Y position';
	const SEM_RANKING_SCALE		= '007d. RANKING : scale';
	
	// IGNORED_PLAYERS_LIST parameter
	const IGNORED_PLAYERS_LIST	= '008. IGNORE_LIST';
	
	// PLAYER_CONNECTED parameters
	const PLAYER_CONNECTED_ENABLED	= '009. Sound PLAYER_CONNECTED enabled ?';
	public $PLAYER_CONNECTED_LINK;
	public $PLAYER_CONNECTED_BLOCKOTHERSOUNDS;
	public $PLAYER_CONNECTED_DURATION;
	
	
	// PLAYER_DISCONNECTED parameters
	const PLAYER_DISCONNECTED_ENABLED	= '010. Sound PLAYER_DISCONNECTED enabled ?';
	public $PLAYER_DISCONNECTED_LINK;
	public $PLAYER_DISCONNECTED_BLOCKOTHERSOUNDS;
	public $PLAYER_DISCONNECTED_DURATION;
	
	// BEGIN_MAP parameters
	const BEGIN_MAP_ENABLED	= '011a. Sound BEGIN_MAP enabled ?';
	public $BEGIN_MAP_LINK;
	public $BEGIN_MAP_BLOCKOTHERSOUNDS;
	public $BEGIN_MAP_DURATION;
	const BEGIN_MAP_DELAY	= '011b. BEGIN_MAP delay in ms';
	
	// BEGIN_PLAYING parameters
	const BEGIN_PLAYING_ENABLED	= '012a. Sound BEGIN_PLAYING enabled ?';
	public $BEGIN_PLAYING_LINK;
	public $BEGIN_PLAYING_BLOCKOTHERSOUNDS;
	public $BEGIN_PLAYING_DURATION;
	const BEGIN_PLAYING_DELAY	= '012b. BEGIN_PLAYING delay in ms';
	
	// BLUE/RED TEAM SCORES parameters
	const TEAM_SCORES_ENABLED_TO	= '013a. Sound TEAM_SCORES enabled to [none/player/team/all] ?';
	public $BLUE_TEAM_SCORES_LINK;
	public $BLUE_TEAM_SCORES_BLOCKOTHERSOUNDS;
	public $BLUE_TEAM_SCORES__DURATION;
	public $RED_TEAM_SCORES_LINK;
	public $RED_TEAM_SCORES_BLOCKOTHERSOUNDS;
	public $RED_TEAM_SCORES_DURATION;
	const TEAM_SCORES_DELAY			= '013b. TEAM_SCORES delay in ms';
	
	// BACKSPACE parameters
	const BACKSPACE_ENABLED_TO	= '014. Sound BACKSPACE enabled to [none/player/team/all] ?';
	public $BACKSPACE_LINK;
	public $BACKSPACE_BLOCKOTHERSOUNDS;
	public $BACKSPACE_DURATION;
	
	// OFFZONE parameters
	const OFFZONE_ENABLED_TO	= '015. Sound OFF-ZONE enabled to [none/player/team/all] ?';
	public $OFFZONE_LINK;
	public $OFFZONE_BLOCKOTHERSOUNDS;
	public $OFFZONE_DURATION;
	
	// HIT parameters
	const HIT_LASER_ENABLED_TO		= '016a. Sound HIT_LASER enabled to [none/player/team/all] ?';
	public $HIT_LASER_LINK;
	public $HIT_LASER_BLOCKOTHERSOUNDS;
	public $HIT_LASER_DURATION;
	const HIT_ROCKET_ENABLED_TO		= '016b. Sound HIT_ROCKET enabled to [none/player/team/all] ?';
	public $HIT_ROCKET_LINK;
	public $HIT_ROCKET_BLOCKOTHERSOUNDS;
	public $HIT_ROCKET_DURATION;
	const HIT_NUCLEUS_ENABLED_TO	= '016c. Sound HIT_NUCLEUS enabled to [none/player/team/all] ?';
	public $HIT_NUCLEUS_LINK;
	public $HIT_NUCLEUS_BLOCKOTHERSOUNDS;
	public $HIT_NUCLEUS_DURATION;
	const HIT_ARROW_ENABLED_TO		= '016d. Sound HIT_ARROW enabled to [none/player/team/all] ?';
	public $HIT_ARROW_LINK;
	public $HIT_ARROW_BLOCKOTHERSOUNDS;
	public $HIT_ARROW_DURATION;
	
	// TEAMKILLER parameters
	const TEAMKILLER_ENABLED_TO	= '017. Sound TEAMKILLER enabled to [none/player/team/all] ?';
	public $TEAMKILLER_LINK;
	public $TEAMKILLER_BLOCKOTHERSOUNDS;
	public $TEAMKILLER_DURATION;
	
	// ONNEARMISS parameters
	// V1.08 const ONNEARMISS_ENABLED_TO	= '018a. ONNEARMISS enabled to [none/player/team/all] ?';
	// V1.08 const ONNEARMISS_LINK			= '018b. ONNEARMISS sound';
	
	// BEGINWARMUP parameters
	// V1.08 const BEGINWARMUP_ENABLED	= '019a. BEGINWARMUP enabled ?';
	// V1.08 const BEGINWARMUP_LINK		= '019b. BEGINWARMUP sound';
	
	// ENDWARMUP parameters
	// V1.08 const ENDWARMUP_ENABLED	= '020a. ENDWARMUP enabled ?';
	// V1.08 const ENDWARMUP_LINK		= '020b. ENDWARMUP sound';
	
	// ENDPLAYING parameters
	// V1.08 const ENDPLAYING_ENABLED	= '021a. ENDPLAYING enabled ?';
	// V1.08 const ENDPLAYING_LINK		= '021b. ENDPLAYING sound';
	
	// ENDMAP parameters
	// V1.08 const ENDMAP_ENABLED	= '022a. ENDMAP enabled ?';
	// V1.08 const ENDMAP_LINK		= '022b. ENDMAP sound';
	
	// KILL AWARDS allow
	const KILL_AWARDS_ENABLED_TO	= '100a. KILLSTREAK sounds enabled to [none/player/team/all] ?';
	
	// RESET_ON_NEW_ROUND parameter
	const RESET_ON_NEW_ROUND	= '100b. Reset KILLSTREAK on new round ?';
	
	// KILL AWARDS parameters
	public $KILL_01_LINK; public $KILL_01_BLOCKOTHERSOUNDS; public $KILL_01_DURATION;
	public $KILL_02_LINK; public $KILL_02_BLOCKOTHERSOUNDS; public $KILL_02_DURATION;
	public $KILL_03_LINK; public $KILL_03_BLOCKOTHERSOUNDS; public $KILL_03_DURATION;
	public $KILL_04_LINK; public $KILL_04_BLOCKOTHERSOUNDS; public $KILL_04_DURATION;
	public $KILL_05_LINK; public $KILL_05_BLOCKOTHERSOUNDS; public $KILL_05_DURATION;
	public $KILL_06_LINK; public $KILL_06_BLOCKOTHERSOUNDS; public $KILL_06_DURATION;
	public $KILL_07_LINK; public $KILL_07_BLOCKOTHERSOUNDS; public $KILL_07_DURATION;
	public $KILL_08_LINK; public $KILL_08_BLOCKOTHERSOUNDS; public $KILL_08_DURATION;
	public $KILL_09_LINK; public $KILL_09_BLOCKOTHERSOUNDS; public $KILL_09_DURATION;
	public $KILL_10_LINK; public $KILL_10_BLOCKOTHERSOUNDS; public $KILL_10_DURATION;
	public $KILL_11_LINK; public $KILL_11_BLOCKOTHERSOUNDS; public $KILL_11_DURATION;
	public $KILL_12_LINK; public $KILL_12_BLOCKOTHERSOUNDS; public $KILL_12_DURATION;
	public $KILL_13_LINK; public $KILL_13_BLOCKOTHERSOUNDS; public $KILL_13_DURATION;
	public $KILL_14_LINK; public $KILL_14_BLOCKOTHERSOUNDS; public $KILL_14_DURATION;
	public $KILL_15_LINK; public $KILL_15_BLOCKOTHERSOUNDS; public $KILL_15_DURATION;
	public $KILL_16_LINK; public $KILL_16_BLOCKOTHERSOUNDS; public $KILL_16_DURATION;
	public $KILL_17_LINK; public $KILL_17_BLOCKOTHERSOUNDS; public $KILL_17_DURATION;
	public $KILL_18_LINK; public $KILL_18_BLOCKOTHERSOUNDS; public $KILL_18_DURATION;
	public $KILL_19_LINK; public $KILL_19_BLOCKOTHERSOUNDS; public $KILL_19_DURATION;
	public $KILL_20_LINK; public $KILL_20_BLOCKOTHERSOUNDS; public $KILL_20_DURATION;
	public $KILL_21_LINK; public $KILL_21_BLOCKOTHERSOUNDS; public $KILL_21_DURATION;
	public $KILL_22_LINK; public $KILL_22_BLOCKOTHERSOUNDS; public $KILL_22_DURATION;
	public $KILL_23_LINK; public $KILL_23_BLOCKOTHERSOUNDS; public $KILL_23_DURATION;
	public $KILL_24_LINK; public $KILL_24_BLOCKOTHERSOUNDS; public $KILL_24_DURATION;
	public $KILL_25_LINK; public $KILL_25_BLOCKOTHERSOUNDS; public $KILL_25_DURATION;
	public $KILL_26_LINK; public $KILL_26_BLOCKOTHERSOUNDS; public $KILL_26_DURATION;
	public $KILL_27_LINK; public $KILL_27_BLOCKOTHERSOUNDS; public $KILL_27_DURATION;
	public $KILL_28_LINK; public $KILL_28_BLOCKOTHERSOUNDS; public $KILL_28_DURATION;
	public $KILL_29_LINK; public $KILL_29_BLOCKOTHERSOUNDS; public $KILL_29_DURATION;
	public $KILL_30_LINK; public $KILL_30_BLOCKOTHERSOUNDS; public $KILL_30_DURATION;
	
	// SOUND ON DEMAND parameters from 01 to 99
	// TO DO ? Créer ces variables dans le load(), en utilisant __set(). Cf http://stackoverflow.com/questions/12330341/php-trying-to-create-dynamic-variables-in-classes
	// le script fonctionne sans la déclaration de ces variables !! php semble gérer la création à la volée, lors de l'initialisation d'une variable
	public $SOUND_01_LINK;	public $SOUND_02_LINK;	public $SOUND_03_LINK;	public $SOUND_04_LINK;	public $SOUND_05_LINK;	public $SOUND_06_LINK;	public $SOUND_07_LINK;	public $SOUND_08_LINK;	public $SOUND_09_LINK;	public $SOUND_10_LINK;
	public $SOUND_11_LINK; 	public $SOUND_12_LINK;	public $SOUND_13_LINK;	public $SOUND_14_LINK;	public $SOUND_15_LINK;	public $SOUND_16_LINK;	public $SOUND_17_LINK;	public $SOUND_18_LINK;	public $SOUND_19_LINK;	public $SOUND_20_LINK;
	public $SOUND_21_LINK;	public $SOUND_22_LINK;	public $SOUND_23_LINK;	public $SOUND_24_LINK;	public $SOUND_25_LINK;	public $SOUND_26_LINK;	public $SOUND_27_LINK;	public $SOUND_28_LINK;	public $SOUND_29_LINK;	public $SOUND_30_LINK;
	public $SOUND_31_LINK;	public $SOUND_32_LINK;	public $SOUND_33_LINK;	public $SOUND_34_LINK;	public $SOUND_35_LINK;	public $SOUND_36_LINK;	public $SOUND_37_LINK;	public $SOUND_38_LINK;	public $SOUND_39_LINK;	public $SOUND_40_LINK;
	public $SOUND_41_LINK; 	public $SOUND_42_LINK;	public $SOUND_43_LINK;	public $SOUND_44_LINK;	public $SOUND_45_LINK;	public $SOUND_46_LINK;	public $SOUND_47_LINK;	public $SOUND_48_LINK;	public $SOUND_49_LINK;	public $SOUND_50_LINK;
	public $SOUND_51_LINK;	public $SOUND_52_LINK;	public $SOUND_53_LINK;	public $SOUND_54_LINK;	public $SOUND_55_LINK;	public $SOUND_56_LINK;	public $SOUND_57_LINK;	public $SOUND_58_LINK;	public $SOUND_59_LINK;	public $SOUND_60_LINK;
	public $SOUND_61_LINK;	public $SOUND_62_LINK;	public $SOUND_63_LINK;	public $SOUND_64_LINK;	public $SOUND_65_LINK;	public $SOUND_66_LINK;	public $SOUND_67_LINK;	public $SOUND_68_LINK;	public $SOUND_69_LINK;	public $SOUND_70_LINK;
	public $SOUND_71_LINK;	public $SOUND_72_LINK;	public $SOUND_73_LINK;	public $SOUND_74_LINK;	public $SOUND_75_LINK;	public $SOUND_76_LINK;	public $SOUND_77_LINK;	public $SOUND_78_LINK;	public $SOUND_79_LINK;	public $SOUND_80_LINK;
	public $SOUND_81_LINK; 	public $SOUND_82_LINK;	public $SOUND_83_LINK;	public $SOUND_84_LINK;	public $SOUND_85_LINK;	public $SOUND_86_LINK;	public $SOUND_87_LINK;	public $SOUND_88_LINK;	public $SOUND_89_LINK;	public $SOUND_90_LINK;
	public $SOUND_91_LINK;	public $SOUND_92_LINK;	public $SOUND_93_LINK;	public $SOUND_94_LINK;	public $SOUND_95_LINK;	public $SOUND_96_LINK;	public $SOUND_97_LINK;	public $SOUND_98_LINK;	public $SOUND_99_LINK;
	public $SOUND_01_NAME;	public $SOUND_02_NAME;	public $SOUND_03_NAME;	public $SOUND_04_NAME;	public $SOUND_05_NAME;	public $SOUND_06_NAME;	public $SOUND_07_NAME;	public $SOUND_08_NAME;	public $SOUND_09_NAME;	public $SOUND_10_NAME;
	public $SOUND_11_NAME; 	public $SOUND_12_NAME;	public $SOUND_13_NAME;	public $SOUND_14_NAME;	public $SOUND_15_NAME;	public $SOUND_16_NAME;	public $SOUND_17_NAME;	public $SOUND_18_NAME;	public $SOUND_19_NAME;	public $SOUND_20_NAME;
	public $SOUND_21_NAME;	public $SOUND_22_NAME;	public $SOUND_23_NAME;	public $SOUND_24_NAME;	public $SOUND_25_NAME;	public $SOUND_26_NAME;	public $SOUND_27_NAME;	public $SOUND_28_NAME;	public $SOUND_29_NAME;	public $SOUND_30_NAME;
	public $SOUND_31_NAME;	public $SOUND_32_NAME;	public $SOUND_33_NAME;	public $SOUND_34_NAME;	public $SOUND_35_NAME;	public $SOUND_36_NAME;	public $SOUND_37_NAME;	public $SOUND_38_NAME;	public $SOUND_39_NAME;	public $SOUND_40_NAME;
	public $SOUND_41_NAME; 	public $SOUND_42_NAME;	public $SOUND_43_NAME;	public $SOUND_44_NAME;	public $SOUND_45_NAME;	public $SOUND_46_NAME;	public $SOUND_47_NAME;	public $SOUND_48_NAME;	public $SOUND_49_NAME;	public $SOUND_50_NAME;
	public $SOUND_51_NAME;	public $SOUND_52_NAME;	public $SOUND_53_NAME;	public $SOUND_54_NAME;	public $SOUND_55_NAME;	public $SOUND_56_NAME;	public $SOUND_57_NAME;	public $SOUND_58_NAME;	public $SOUND_59_NAME;	public $SOUND_60_NAME;
	public $SOUND_61_NAME;	public $SOUND_62_NAME;	public $SOUND_63_NAME;	public $SOUND_64_NAME;	public $SOUND_65_NAME;	public $SOUND_66_NAME;	public $SOUND_67_NAME;	public $SOUND_68_NAME;	public $SOUND_69_NAME;	public $SOUND_70_NAME;
	public $SOUND_71_NAME;	public $SOUND_72_NAME;	public $SOUND_73_NAME;	public $SOUND_74_NAME;	public $SOUND_75_NAME;	public $SOUND_76_NAME;	public $SOUND_77_NAME;	public $SOUND_78_NAME;	public $SOUND_79_NAME;	public $SOUND_80_NAME;
	public $SOUND_81_NAME; 	public $SOUND_82_NAME;	public $SOUND_83_NAME;	public $SOUND_84_NAME;	public $SOUND_85_NAME;	public $SOUND_86_NAME;	public $SOUND_87_NAME;	public $SOUND_88_NAME;	public $SOUND_89_NAME;	public $SOUND_90_NAME;
	public $SOUND_91_NAME;	public $SOUND_92_NAME;	public $SOUND_93_NAME;	public $SOUND_94_NAME;	public $SOUND_95_NAME;	public $SOUND_96_NAME;	public $SOUND_97_NAME;	public $SOUND_98_NAME;	public $SOUND_99_NAME;
	public $SOUND_01_DESCRIPTION;	public $SOUND_02_DESCRIPTION;	public $SOUND_03_DESCRIPTION;	public $SOUND_04_DESCRIPTION;	public $SOUND_05_DESCRIPTION;	public $SOUND_06_DESCRIPTION;	public $SOUND_07_DESCRIPTION;	public $SOUND_08_DESCRIPTION;	public $SOUND_09_DESCRIPTION;	public $SOUND_10_DESCRIPTION;
	public $SOUND_11_DESCRIPTION; 	public $SOUND_12_DESCRIPTION;	public $SOUND_13_DESCRIPTION;	public $SOUND_14_DESCRIPTION;	public $SOUND_15_DESCRIPTION;	public $SOUND_16_DESCRIPTION;	public $SOUND_17_DESCRIPTION;	public $SOUND_18_DESCRIPTION;	public $SOUND_19_DESCRIPTION;	public $SOUND_20_DESCRIPTION;
	public $SOUND_21_DESCRIPTION;	public $SOUND_22_DESCRIPTION;	public $SOUND_23_DESCRIPTION;	public $SOUND_24_DESCRIPTION;	public $SOUND_25_DESCRIPTION;	public $SOUND_26_DESCRIPTION;	public $SOUND_27_DESCRIPTION;	public $SOUND_28_DESCRIPTION;	public $SOUND_29_DESCRIPTION;	public $SOUND_30_DESCRIPTION;
	public $SOUND_31_DESCRIPTION;	public $SOUND_32_DESCRIPTION;	public $SOUND_33_DESCRIPTION;	public $SOUND_34_DESCRIPTION;	public $SOUND_35_DESCRIPTION;	public $SOUND_36_DESCRIPTION;	public $SOUND_37_DESCRIPTION;	public $SOUND_38_DESCRIPTION;	public $SOUND_39_DESCRIPTION;	public $SOUND_40_DESCRIPTION;
	public $SOUND_41_DESCRIPTION; 	public $SOUND_42_DESCRIPTION;	public $SOUND_43_DESCRIPTION;	public $SOUND_44_DESCRIPTION;	public $SOUND_45_DESCRIPTION;	public $SOUND_46_DESCRIPTION;	public $SOUND_47_DESCRIPTION;	public $SOUND_48_DESCRIPTION;	public $SOUND_49_DESCRIPTION;	public $SOUND_50_DESCRIPTION;
	public $SOUND_51_DESCRIPTION;	public $SOUND_52_DESCRIPTION;	public $SOUND_53_DESCRIPTION;	public $SOUND_54_DESCRIPTION;	public $SOUND_55_DESCRIPTION;	public $SOUND_56_DESCRIPTION;	public $SOUND_57_DESCRIPTION;	public $SOUND_58_DESCRIPTION;	public $SOUND_59_DESCRIPTION;	public $SOUND_60_DESCRIPTION;
	public $SOUND_61_DESCRIPTION;	public $SOUND_62_DESCRIPTION;	public $SOUND_63_DESCRIPTION;	public $SOUND_64_DESCRIPTION;	public $SOUND_65_DESCRIPTION;	public $SOUND_66_DESCRIPTION;	public $SOUND_67_DESCRIPTION;	public $SOUND_68_DESCRIPTION;	public $SOUND_69_DESCRIPTION;	public $SOUND_70_DESCRIPTION;
	public $SOUND_71_DESCRIPTION;	public $SOUND_72_DESCRIPTION;	public $SOUND_73_DESCRIPTION;	public $SOUND_74_DESCRIPTION;	public $SOUND_75_DESCRIPTION;	public $SOUND_76_DESCRIPTION;	public $SOUND_77_DESCRIPTION;	public $SOUND_78_DESCRIPTION;	public $SOUND_79_DESCRIPTION;	public $SOUND_80_DESCRIPTION;
	public $SOUND_81_DESCRIPTION; 	public $SOUND_82_DESCRIPTION;	public $SOUND_83_DESCRIPTION;	public $SOUND_84_DESCRIPTION;	public $SOUND_85_DESCRIPTION;	public $SOUND_86_DESCRIPTION;	public $SOUND_87_DESCRIPTION;	public $SOUND_88_DESCRIPTION;	public $SOUND_89_DESCRIPTION;	public $SOUND_90_DESCRIPTION;
	public $SOUND_91_DESCRIPTION;	public $SOUND_92_DESCRIPTION;	public $SOUND_93_DESCRIPTION;	public $SOUND_94_DESCRIPTION;	public $SOUND_95_DESCRIPTION;	public $SOUND_96_DESCRIPTION;	public $SOUND_97_DESCRIPTION;	public $SOUND_98_DESCRIPTION;	public $SOUND_99_DESCRIPTION;
	public $SOUND_01_BLOCKOTHERSOUNDS;	public $SOUND_02_BLOCKOTHERSOUNDS;	public $SOUND_03_BLOCKOTHERSOUNDS;	public $SOUND_04_BLOCKOTHERSOUNDS;	public $SOUND_05_BLOCKOTHERSOUNDS;	public $SOUND_06_BLOCKOTHERSOUNDS;	public $SOUND_07_BLOCKOTHERSOUNDS;	public $SOUND_08_BLOCKOTHERSOUNDS;	public $SOUND_09_BLOCKOTHERSOUNDS;	public $SOUND_10_BLOCKOTHERSOUNDS;
	public $SOUND_11_BLOCKOTHERSOUNDS; 	public $SOUND_12_BLOCKOTHERSOUNDS;	public $SOUND_13_BLOCKOTHERSOUNDS;	public $SOUND_14_BLOCKOTHERSOUNDS;	public $SOUND_15_BLOCKOTHERSOUNDS;	public $SOUND_16_BLOCKOTHERSOUNDS;	public $SOUND_17_BLOCKOTHERSOUNDS;	public $SOUND_18_BLOCKOTHERSOUNDS;	public $SOUND_19_BLOCKOTHERSOUNDS;	public $SOUND_20_BLOCKOTHERSOUNDS;
	public $SOUND_21_BLOCKOTHERSOUNDS;	public $SOUND_22_BLOCKOTHERSOUNDS;	public $SOUND_23_BLOCKOTHERSOUNDS;	public $SOUND_24_BLOCKOTHERSOUNDS;	public $SOUND_25_BLOCKOTHERSOUNDS;	public $SOUND_26_BLOCKOTHERSOUNDS;	public $SOUND_27_BLOCKOTHERSOUNDS;	public $SOUND_28_BLOCKOTHERSOUNDS;	public $SOUND_29_BLOCKOTHERSOUNDS;	public $SOUND_30_BLOCKOTHERSOUNDS;
	public $SOUND_31_BLOCKOTHERSOUNDS;	public $SOUND_32_BLOCKOTHERSOUNDS;	public $SOUND_33_BLOCKOTHERSOUNDS;	public $SOUND_34_BLOCKOTHERSOUNDS;	public $SOUND_35_BLOCKOTHERSOUNDS;	public $SOUND_36_BLOCKOTHERSOUNDS;	public $SOUND_37_BLOCKOTHERSOUNDS;	public $SOUND_38_BLOCKOTHERSOUNDS;	public $SOUND_39_BLOCKOTHERSOUNDS;	public $SOUND_40_BLOCKOTHERSOUNDS;
	public $SOUND_41_BLOCKOTHERSOUNDS; 	public $SOUND_42_BLOCKOTHERSOUNDS;	public $SOUND_43_BLOCKOTHERSOUNDS;	public $SOUND_44_BLOCKOTHERSOUNDS;	public $SOUND_45_BLOCKOTHERSOUNDS;	public $SOUND_46_BLOCKOTHERSOUNDS;	public $SOUND_47_BLOCKOTHERSOUNDS;	public $SOUND_48_BLOCKOTHERSOUNDS;	public $SOUND_49_BLOCKOTHERSOUNDS;	public $SOUND_50_BLOCKOTHERSOUNDS;
	public $SOUND_51_BLOCKOTHERSOUNDS;	public $SOUND_52_BLOCKOTHERSOUNDS;	public $SOUND_53_BLOCKOTHERSOUNDS;	public $SOUND_54_BLOCKOTHERSOUNDS;	public $SOUND_55_BLOCKOTHERSOUNDS;	public $SOUND_56_BLOCKOTHERSOUNDS;	public $SOUND_57_BLOCKOTHERSOUNDS;	public $SOUND_58_BLOCKOTHERSOUNDS;	public $SOUND_59_BLOCKOTHERSOUNDS;	public $SOUND_60_BLOCKOTHERSOUNDS;
	public $SOUND_61_BLOCKOTHERSOUNDS;	public $SOUND_62_BLOCKOTHERSOUNDS;	public $SOUND_63_BLOCKOTHERSOUNDS;	public $SOUND_64_BLOCKOTHERSOUNDS;	public $SOUND_65_BLOCKOTHERSOUNDS;	public $SOUND_66_BLOCKOTHERSOUNDS;	public $SOUND_67_BLOCKOTHERSOUNDS;	public $SOUND_68_BLOCKOTHERSOUNDS;	public $SOUND_69_BLOCKOTHERSOUNDS;	public $SOUND_70_BLOCKOTHERSOUNDS;
	public $SOUND_71_BLOCKOTHERSOUNDS;	public $SOUND_72_BLOCKOTHERSOUNDS;	public $SOUND_73_BLOCKOTHERSOUNDS;	public $SOUND_74_BLOCKOTHERSOUNDS;	public $SOUND_75_BLOCKOTHERSOUNDS;	public $SOUND_76_BLOCKOTHERSOUNDS;	public $SOUND_77_BLOCKOTHERSOUNDS;	public $SOUND_78_BLOCKOTHERSOUNDS;	public $SOUND_79_BLOCKOTHERSOUNDS;	public $SOUND_80_BLOCKOTHERSOUNDS;
	public $SOUND_81_BLOCKOTHERSOUNDS; 	public $SOUND_82_BLOCKOTHERSOUNDS;	public $SOUND_83_BLOCKOTHERSOUNDS;	public $SOUND_84_BLOCKOTHERSOUNDS;	public $SOUND_85_BLOCKOTHERSOUNDS;	public $SOUND_86_BLOCKOTHERSOUNDS;	public $SOUND_87_BLOCKOTHERSOUNDS;	public $SOUND_88_BLOCKOTHERSOUNDS;	public $SOUND_89_BLOCKOTHERSOUNDS;	public $SOUND_90_BLOCKOTHERSOUNDS;
	public $SOUND_91_BLOCKOTHERSOUNDS;	public $SOUND_92_BLOCKOTHERSOUNDS;	public $SOUND_93_BLOCKOTHERSOUNDS;	public $SOUND_94_BLOCKOTHERSOUNDS;	public $SOUND_95_BLOCKOTHERSOUNDS;	public $SOUND_96_BLOCKOTHERSOUNDS;	public $SOUND_97_BLOCKOTHERSOUNDS;	public $SOUND_98_BLOCKOTHERSOUNDS;	public $SOUND_99_BLOCKOTHERSOUNDS;
	public $SOUND_01_DURATION;	public $SOUND_02_DURATION;	public $SOUND_03_DURATION;	public $SOUND_04_DURATION;	public $SOUND_05_DURATION;	public $SOUND_06_DURATION;	public $SOUND_07_DURATION;	public $SOUND_08_DURATION;	public $SOUND_09_DURATION;	public $SOUND_10_DURATION;
	public $SOUND_11_DURATION; 	public $SOUND_12_DURATION;	public $SOUND_13_DURATION;	public $SOUND_14_DURATION;	public $SOUND_15_DURATION;	public $SOUND_16_DURATION;	public $SOUND_17_DURATION;	public $SOUND_18_DURATION;	public $SOUND_19_DURATION;	public $SOUND_20_DURATION;
	public $SOUND_21_DURATION;	public $SOUND_22_DURATION;	public $SOUND_23_DURATION;	public $SOUND_24_DURATION;	public $SOUND_25_DURATION;	public $SOUND_26_DURATION;	public $SOUND_27_DURATION;	public $SOUND_28_DURATION;	public $SOUND_29_DURATION;	public $SOUND_30_DURATION;
	public $SOUND_31_DURATION;	public $SOUND_32_DURATION;	public $SOUND_33_DURATION;	public $SOUND_34_DURATION;	public $SOUND_35_DURATION;	public $SOUND_36_DURATION;	public $SOUND_37_DURATION;	public $SOUND_38_DURATION;	public $SOUND_39_DURATION;	public $SOUND_40_DURATION;
	public $SOUND_41_DURATION; 	public $SOUND_42_DURATION;	public $SOUND_43_DURATION;	public $SOUND_44_DURATION;	public $SOUND_45_DURATION;	public $SOUND_46_DURATION;	public $SOUND_47_DURATION;	public $SOUND_48_DURATION;	public $SOUND_49_DURATION;	public $SOUND_50_DURATION;
	public $SOUND_51_DURATION;	public $SOUND_52_DURATION;	public $SOUND_53_DURATION;	public $SOUND_54_DURATION;	public $SOUND_55_DURATION;	public $SOUND_56_DURATION;	public $SOUND_57_DURATION;	public $SOUND_58_DURATION;	public $SOUND_59_DURATION;	public $SOUND_60_DURATION;
	public $SOUND_61_DURATION;	public $SOUND_62_DURATION;	public $SOUND_63_DURATION;	public $SOUND_64_DURATION;	public $SOUND_65_DURATION;	public $SOUND_66_DURATION;	public $SOUND_67_DURATION;	public $SOUND_68_DURATION;	public $SOUND_69_DURATION;	public $SOUND_70_DURATION;
	public $SOUND_71_DURATION;	public $SOUND_72_DURATION;	public $SOUND_73_DURATION;	public $SOUND_74_DURATION;	public $SOUND_75_DURATION;	public $SOUND_76_DURATION;	public $SOUND_77_DURATION;	public $SOUND_78_DURATION;	public $SOUND_79_DURATION;	public $SOUND_80_DURATION;
	public $SOUND_81_DURATION; 	public $SOUND_82_DURATION;	public $SOUND_83_DURATION;	public $SOUND_84_DURATION;	public $SOUND_85_DURATION;	public $SOUND_86_DURATION;	public $SOUND_87_DURATION;	public $SOUND_88_DURATION;	public $SOUND_89_DURATION;	public $SOUND_90_DURATION;
	public $SOUND_91_DURATION;	public $SOUND_92_DURATION;	public $SOUND_93_DURATION;	public $SOUND_94_DURATION;	public $SOUND_95_DURATION;	public $SOUND_96_DURATION;	public $SOUND_97_DURATION;	public $SOUND_98_DURATION;	public $SOUND_99_DURATION;
		
	// Secret (non-displayed and non-configurable) parameters to store/load users config
	const DISABLED_MUSIC_PLAYERS			= '666.1 DISABLED_MUSIC_PLAYERS';
	const DISABLED_RANKING_PLAYERS			= '666.2 DISABLED_RANKING_PLAYERS';
	const DISABLED_MESSAGES_PLAYERS			= '666.3 DISABLED_MESSAGES_PLAYERS';
	const DISABLED_SOUNDS_PLAYERS			= '666.4 DISABLED_SOUNDS_PLAYERS';
	const DISABLED_SOUND_ON_DEMAND_PLAYERS	= '666.5 DISABLED_SOUND_ON_DEMAND_PLAYERS';
	
	
    /**
     * Private Properties
     */
    private $maniaControl = null;
	
	// SEM
	public $playersScore = array();
	public $disabledSoundsPlayers = array();
	public $disabledMessagesPlayers = array();
	public $disabledRankingPlayers = array();
	public $disabledMusicPlayers = array();
	public $disabledSoundOnDemandPlayers = array();
	public $SOD_spamList = array();
	public $playerTeamId;
	public $scoreMin;
	public $soundLinkMin;
	public $scoreMax;
	public $soundLinkMax;
	public $musicIsPlaying = false;
	public $SOD_global_delay_elapsed = true;
	public $soundsBlocked = false;
	
	
    /**
     * Fonction de log pour debug
     *
     * @param $log
     */
    public function toLog($log) {
		if ($this->maniaControl->settingManager->getSettingValue($this, self::DEBUG)) {
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager][' . $this->getVersion() . '] ' . $log, null, false);
			$this->sendInformation($log, null);
		}
    }
	
	
	
	/*
	 * Initialize the array which stores players' scores
	 */
	public function initializePlayersScore() {
		// Initialisation of playersScore[]
		foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			$playerLogin = $player->login;
			$this->playersScore[$playerLogin] = '0';
		}
	}
	
	
	
    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     */
    public static function prepare(ManiaControl $maniaControl) {
		//$this->toLog('into prepare()');
		// do nothing
    }
	
	
	
	/**
	 * Unload the plugin and its resources
	 */
    public function unload() {
		// DEBUG
		$this->toLog('Unloading Sound Events Manager...');
		
		// Reset arrays
		$this->playersScore = array();
		$this->disabledSoundsPlayers = array();
		$this->disabledMessagesPlayers = array();
		$this->disabledRankingPlayers = array();
		$this->disabledMusicPlayers = array();
		$this->disabledSoundOnDemandPlayers = array();
		
		// Removing SEM Icon
		$this->removeMenuIcon();
		
		// Update ranking
		$this->ranking();
		
		// Callbacks
        $this->maniaControl->getCallbackManager()->unregisterCallbackListener($this);
		
		// ManialinkPageAnswerListener
		$this->maniaControl->manialinkManager->unregisterManialinkPageAnswerListener($this);
		
		// Unset maniaControl
		// TO DO : soit ca bug pcq getChat() n'est pas reconnu (si unset), soit j'ai 2 fois les messages car 2 fois $maniaControl apparemment
		// KO : problème pas corrigé par ligne au dessus (unregisterManialinkPageAnswerListener) ?
        //unset($this->maniaControl);
		//$this->maniaControl = null;
    }
	
    /**
     * Load the plugin
     *
     * @param \ManiaControl\ManiaControl $maniaControl
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;
		
		$this->toLog('Loading Sound Events Manager...');
		
		// Detecting mode		
		// Cf http://mania-actu.com/browser/servers.php
		//    'SMStormElite@nadeolabs'
		//    'SMStormRoyal@nadeolabs'
		//    'SMStormSiege@nadeolabs'
		//    'SMStormJoust@nadeolabs'
		//    'SMStormCombo@nadeolabs'
		//    'SMStorm' (Battle / Instagib)
		//    'BattlePro@jousticeleague'
		//    'SpeedBall@steeffeen'
		//    'Obstacle@steeffeen'
		//    'CaptureTheFlag@mewin'
		$mode = $this->maniaControl->server->titleId;
		$this->toLog("Mode is $mode");
		$serverName = $this->maniaControl->server->login;
		$serverIp = $this->maniaControl->server->ip;
		$this->toLog("Server name is $serverName ($serverIp)");
		
		// Declare KILL_*_LINK constants
		// TO DO : gérer la création et utilisation des KILL_*_LINK
		// TO DO : supprimer les anciens paramètres des KILL_*_LINK
		/*
		for ($i = 1; $i <= 30 ; $i++) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// Define variable KILL_*_LINK
			public ${"KILL_" . $i_2digits . "_LINK"};
		}
		*/
		
		// GLOBAL VARIABLES
		$rep_sounds = 'http://mm2monde.fr/Plugins/SoundEventsManager/UnrealTournamentSounds/';
		$this->scoreMin = 666;
		$this->soundLinkMin = '';
		$this->scoreMax = 666;;
		$this->soundLinkMax = '';
		$this->disabledSoundsPlayers = array();
		$this->disabledMessagesPlayers = array();
		$this->disabledRankingPlayers = array();
		$this->disabledMusicPlayers = array();
		$this->disabledSoundOnDemandPlayers = array();
		
		// XML configuration file
		$this->CONFIG_XML					= __DIR__ . DIRECTORY_SEPARATOR . 'config.xml';
		// XML default configuration file
		$this->CONFIG_XML_DEFAULT			= __DIR__ . DIRECTORY_SEPARATOR . 'config_default.xml';
		
		// XML sounds on demand file
		$this->SOUND_ON_DEMAND_XML			= __DIR__ . DIRECTORY_SEPARATOR . 'soundOnDemand.xml';
		// XML default sounds on demand file
		$this->SOUND_ON_DEMAND_XML_DEFAULT	= __DIR__ . DIRECTORY_SEPARATOR . 'soundOnDemand_default.xml';
		
		// NB : for all variables bellow, using global variable instead of dynamical variables created from CONFIG_XML file because all variables will be used and some dynamical variables can be not created if the CONFIG_XML file is wrong/empty.
		$this->MUSIC_LINK                = '';
		$this->PLAYER_CONNECTED_LINK     = '';
		$this->PLAYER_DISCONNECTED_LINK  = '';
		$this->BEGIN_MAP_LINK            = '';
		$this->BEGIN_PLAYING_LINK        = '';
		$this->BLUE_TEAM_SCORES_LINK     = '';
		$this->RED_TEAM_SCORES_LINK      = '';
		$this->BACKSPACE_LINK            = '';
		$this->OFFZONE_LINK              = '';
		$this->HIT_LASER_LINK            = '';
		$this->HIT_ROCKET_LINK           = '';
		$this->HIT_NUCLEUS_LINK          = '';
		$this->HIT_ARROW_LINK            = '';
		$this->TEAMKILLER_LINK           = '';
		$this->KILL_01_LINK              = '';
		$this->KILL_02_LINK              = '';
		$this->KILL_03_LINK              = '';
		$this->KILL_04_LINK              = '';
		$this->KILL_05_LINK              = '';
		$this->KILL_06_LINK              = '';
		$this->KILL_07_LINK              = '';
		$this->KILL_08_LINK              = '';
		$this->KILL_09_LINK              = '';
		$this->KILL_10_LINK              = '';
		$this->KILL_11_LINK              = '';
		$this->KILL_12_LINK              = '';
		$this->KILL_13_LINK              = '';
		$this->KILL_14_LINK              = '';
		$this->KILL_15_LINK              = '';
		$this->KILL_16_LINK              = '';
		$this->KILL_17_LINK              = '';
		$this->KILL_18_LINK              = '';
		$this->KILL_19_LINK              = '';
		$this->KILL_20_LINK              = '';
		$this->KILL_21_LINK              = '';
		$this->KILL_22_LINK              = '';
		$this->KILL_23_LINK              = '';
		$this->KILL_24_LINK              = '';
		$this->KILL_25_LINK              = '';
		$this->KILL_26_LINK              = '';
		$this->KILL_27_LINK              = '';
		$this->KILL_28_LINK              = '';
		$this->KILL_29_LINK              = '';
		$this->KILL_30_LINK              = '';
		
		// RESET
		/*
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_SOUNDS_PLAYERS');
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_MESSAGES_PLAYERS');
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_RANKING_PLAYERS');
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_MUSIC_PLAYERS');
		*/
		
		// Get user parameters from settings
		$LIST_DISABLED_SOUNDS_PLAYERS			= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_SOUNDS_PLAYERS);
		$LIST_DISABLED_MESSAGES_PLAYERS			= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_MESSAGES_PLAYERS);
		$LIST_DISABLED_RANKING_PLAYERS			= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_RANKING_PLAYERS);
		$LIST_DISABLED_MUSIC_PLAYERS			= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_MUSIC_PLAYERS);
		$LIST_DISABLED_SOUND_ON_DEMAND_PLAYERS	= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_SOUND_ON_DEMAND_PLAYERS);
		
		// Store settings into global arrays
		if (!empty($LIST_DISABLED_SOUNDS_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_SOUNDS_PLAYERS) as $element) {
				array_push($this->disabledSoundsPlayers, $element);
			}
		}
		else { $this->disabledSoundsPlayers = array(); }
		
		if (!empty($LIST_DISABLED_MESSAGES_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_MESSAGES_PLAYERS) as $element) {
				array_push($this->disabledMessagesPlayers, $element);
			}
		}
		else { $this->disabledMessagesPlayers = array(); $this->toLog("--> New disabledMessagesPlayers[]"); }
		
		if (!empty($LIST_DISABLED_RANKING_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_RANKING_PLAYERS) as $element) {
				array_push($this->disabledRankingPlayers, $element);
			}
		}
		else { $this->disabledRankingPlayers = array(); }
		
		if (!empty($LIST_DISABLED_MUSIC_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_MUSIC_PLAYERS) as $element) {
				array_push($this->disabledMusicPlayers, $element);
			}
		}
		else { $this->disabledMusicPlayers = array(); }
		
		if (!empty($LIST_DISABLED_SOUND_ON_DEMAND_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_SOUND_ON_DEMAND_PLAYERS) as $element) {
				array_push($this->disabledSoundOnDemandPlayers, $element);
			}
		}
		else { $this->disabledSoundOnDemandPlayers = array(); }
		
		// Initialisation of players score
		$this->initializePlayersScore();
		
        //Register callbacks
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINMAP, $this, 'onBeginMap');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINPLAYING, $this, 'onBeginPlaying');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONCAPTURE, $this, 'onCapture');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONPLAYERREQUESTRESPAWN, $this, 'onRequestRespawn');
		//$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONSHOOT, $this, 'onShoot');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONHIT, $this, 'onHit');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONARMOREMPTY, $this, 'onArmorEmpty');
		
		// TO TEST
		// V1.08 $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONNEARMISS, $this, 'onNearMiss');
		// V1.08 $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINWARMUP, $this, 'onBeginWarmup');
		// V1.08 $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ENDWARMUP, $this, 'onEndWarmup');
		// V1.08 $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ENDPLAYING, $this, 'onEndPlaying');
		// V1.08 $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ENDMAP, $this, 'onEndMap');
		// KO $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::AFKSTATUS, $this, 'onAfkStatus');
		// KO $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINPODIUM, $this, 'onBeginPodium');
		// KO $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ENDPODIUM, $this, 'onEndPodium'); // TO DO : USELESS ???
		// KO $this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_PLAYERALLIESCHANGED, $this, 'onPlayerAlliesChanged');
		// KO $this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_PLAYERINFOCHANGED, $this, 'onPlayerInfoChanged');
		
		
		$this->maniaControl->getCallbackManager()->registerScriptCallbackListener('SpeedBall_OnPass', $this, 'onPass');
		
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'onPlayerConnect');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'onPlayerDisconnect');
		
		// SOUND ON DEMAND
		// NB : Using registerCallbackListener() instead of registerManialinkPageAnswerListener() to allow to pass parameters in the CallbackName)
		$this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_PLAYERMANIALINKPAGEANSWER, $this, 'playSoundOnDemand');
		
		// DEBUG
		$this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACK, $this, 'callbacksForDebug_array');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACKARRAY, $this, 'callbacksForDebug_array');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::SCRIPTCALLBACK , $this, 'callbacksForDebug_string');
		
		// URL message
		$this->maniaControl->settingManager->initSetting($this, self::URL, '$lhttps://forum.maniaplanet.com/viewtopic.php?f=479&t=30285&p=238505#p238505');
		
		// SOUND ON DEMAND
		$this->maniaControl->settingManager->initSetting($this, self::SEM_SOUND_ON_DEMAND_POS_X, -158);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_SOUND_ON_DEMAND_POS_Y, 33);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_SOUND_ON_DEMAND_GLOBAL_DELAY, 0);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_SOUND_ON_DEMAND_USER_DELAY, 0);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_SOUND_ON_DEMAND_MESSAGE_ACTIVATED, true);
		
		// DEBUG messages
		$this->maniaControl->settingManager->initSetting($this, self::DEBUG, false);
		$this->maniaControl->settingManager->initSetting($this, self::PRELOAD_MESSAGE_ACTIVATED, true);
		
		// VOLUME sound
		$this->maniaControl->settingManager->initSetting($this, self::VOLUME_SOUND, 0.8);
		$this->maniaControl->settingManager->initSetting($this, self::VOLUME_SOUND_ON_DEMAND, 0.8);
		
		// ICON_MENU
		$this->maniaControl->settingManager->initSetting($this, self::SEM_ICON_MENU_POS_X, 156);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_ICON_MENU_POS_Y, -38.5);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_ICON_MENU_WIDTH, 6);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_ICON_MENU_HEIGHT, 6);
		
		// MUSIC Settings
		$this->maniaControl->settingManager->initSetting($this, self::MUSIC_ENABLED, false);
		$this->maniaControl->settingManager->initSetting($this, self::MUSIC_VOLUME, 0.5);
		$this->maniaControl->settingManager->initSetting($this, self::MUSIC_DELAY, 5500);
		
		// MESSAGES
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_POS_X, 0);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_POS_Y, 40);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_SCALE, 1.5);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_DURATION, 3000);
		
		// RANKING
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_POS_X, 156);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_POS_Y, 50);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_SCALE, 1);
		
		// IGNORED_PLAYERS_LIST Setting
		$this->maniaControl->settingManager->initSetting($this, self::IGNORED_PLAYERS_LIST, 'wellef;itzviolent');
		
		// PLAYER_CONNECTED Settings
		$this->maniaControl->settingManager->initSetting($this, self::PLAYER_CONNECTED_ENABLED, false);
		
		// PLAYER_DISCONNECTED Settings
		$this->maniaControl->settingManager->initSetting($this, self::PLAYER_DISCONNECTED_ENABLED, false);
		
		// BEGIN_MAP : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_MAP_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_MAP_DELAY, 3000);
		
		// BEGIN_PLAYING : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_PLAYING_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_PLAYING_DELAY, 3000);
		
		// BLUE/RED TEAM SCORES : allow and links
		$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'all');
		$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_DELAY, 1000);
				
		// BACKSPACE : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::BACKSPACE_ENABLED_TO, 'all');
		
		// OFFZONE : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::OFFZONE_ENABLED_TO, 'all');
		
		// HIT : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::HIT_LASER_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_ROCKET_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_NUCLEUS_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_ARROW_ENABLED_TO, 'none');
		
		// TEAMKILLER : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::TEAMKILLER_ENABLED_TO, 'none');
		
		// ONNEARMISS : allow and link
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ONNEARMISS_ENABLED_TO, 'none');
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ONNEARMISS_LINK, '');
		
		// BEGINWARMUP : allow and link
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::BEGINWARMUP_ENABLED, false);
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::BEGINWARMUP_LINK, '');
		
		// ENDWARMUP : allow and link
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ENDWARMUP_ENABLED, false);
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ENDWARMUP_LINK, '');
		
		// ENDPLAYING : allow and link
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ENDPLAYING_ENABLED, false);
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ENDPLAYING_LINK, '');
		
		// ENDMAP : allow and link
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ENDMAP_ENABLED, false);
		// V1.08 $this->maniaControl->settingManager->initSetting($this, self::ENDMAP_LINK, '');
		
		// KILL AWARDS : allow
		$this->maniaControl->settingManager->initSetting($this, self::KILL_AWARDS_ENABLED_TO, 'all');
		
		// RESET_ON_NEW_ROUND Setting
		$this->maniaControl->settingManager->initSetting($this, self::RESET_ON_NEW_ROUND, false);
		
		/*
		// Toute cette partie semble inutile.
		// Reflexion sur les variables KILL_*_LINK et autres :
		//		les déclarer dans la classe
		//		les initialiser à vide dans le load
		//		ne pas mettre de valeur par défaut, autre que celles du fichier XML. Si pas de valeur dans fichier XML, champ par défaut = '' (valeur d'init)
		// NB : possibilité de ne pas déclarer les variables, juste avoir l'initialisation dans le load. Pas très propre mais fonctionne.
		//
		// MODE specific
		switch ($mode) {
			// SpeedBall default config
			case 'SpeedBall@steeffeen':
				// TEAMKILLER : allow and link
				$this->maniaControl->settingManager->initSetting($this, self::TEAMKILLER_ENABLED_TO, 'all');
				
				// KILL AWARDS : links
				$this->KILL_01_LINK = '';
				$this->KILL_02_LINK = '';
				$this->KILL_03_LINK = $rep_sounds . 'dominating.wav   ';
				$this->KILL_04_LINK = $rep_sounds . 'rampage.wav      ';
				$this->KILL_05_LINK = $rep_sounds . 'killingspree.wav ';
				$this->KILL_06_LINK = $rep_sounds . 'ultrakill.wav    ';
				$this->KILL_07_LINK = $rep_sounds . 'wickedsick.wav   ';
				$this->KILL_08_LINK = $rep_sounds . 'monsterkill.wav  ';
				$this->KILL_09_LINK = $rep_sounds . 'unstoppable.wav  ';
				$this->KILL_10_LINK = $rep_sounds . 'godlike.wav      ';
				$this->KILL_11_LINK = $rep_sounds . 'holyshit.wav     ';
				$this->KILL_12_LINK = $rep_sounds . 'LudicrousKill.wav';
				$this->KILL_13_LINK = '';
				$this->KILL_14_LINK = '';
				$this->KILL_15_LINK = '';
				$this->KILL_16_LINK = '';
				$this->KILL_17_LINK = '';
				$this->KILL_18_LINK = '';
				$this->KILL_19_LINK = '';
				$this->KILL_20_LINK = '';
				$this->KILL_21_LINK = '';
				$this->KILL_22_LINK = '';
				$this->KILL_23_LINK = '';
				$this->KILL_24_LINK = '';
				$this->KILL_25_LINK = '';
				$this->KILL_26_LINK = '';
				$this->KILL_27_LINK = '';
				$this->KILL_28_LINK = '';
				$this->KILL_29_LINK = '';
				$this->KILL_30_LINK = '';
				break;
			case 'SMStormSiege@nadeolabs':
				// BLUE/RED TEAM SCORES : allow
				$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'none');
				
				// RESET_ON_NEW_ROUND Setting
				$this->maniaControl->settingManager->initSetting($this, self::RESET_ON_NEW_ROUND, true);
				
				// KILL AWARDS : links
				$this->KILL_01_LINK = '';
				$this->KILL_02_LINK = $rep_sounds . 'dominating.wav   ';
				$this->KILL_03_LINK = '';
				$this->KILL_04_LINK = $rep_sounds . 'rampage.wav      ';
				$this->KILL_05_LINK = '';
				$this->KILL_06_LINK = $rep_sounds . 'killingspree.wav ';
				$this->KILL_07_LINK = '';
				$this->KILL_08_LINK = $rep_sounds . 'ultrakill.wav    ';
				$this->KILL_09_LINK = '';
				$this->KILL_10_LINK = $rep_sounds . 'wickedsick.wav   ';
				$this->KILL_11_LINK = '';
				$this->KILL_12_LINK = $rep_sounds . 'monsterkill.wav  ';
				$this->KILL_13_LINK = '';
				$this->KILL_14_LINK = $rep_sounds . 'unstoppable.wav  ';
				$this->KILL_15_LINK = '';
				$this->KILL_16_LINK = $rep_sounds . 'godlike.wav      ';
				$this->KILL_17_LINK = '';
				$this->KILL_18_LINK = $rep_sounds . 'holyshit.wav     ';
				$this->KILL_19_LINK = '';
				$this->KILL_20_LINK = $rep_sounds . 'LudicrousKill.wav';
				$this->KILL_21_LINK = '';
				$this->KILL_22_LINK = '';
				$this->KILL_23_LINK = '';
				$this->KILL_24_LINK = '';
				$this->KILL_25_LINK = '';
				$this->KILL_26_LINK = '';
				$this->KILL_27_LINK = '';
				$this->KILL_28_LINK = '';
				$this->KILL_29_LINK = '';
				$this->KILL_30_LINK = '';
				break;
			default:
				// KILL AWARDS : links
				$this->KILL_01_LINK = '';
				$this->KILL_02_LINK = '';
				$this->KILL_03_LINK = $rep_sounds . 'dominating.wav   ';
				$this->KILL_04_LINK = $rep_sounds . 'rampage.wav      ';
				$this->KILL_05_LINK = $rep_sounds . 'killingspree.wav ';
				$this->KILL_06_LINK = $rep_sounds . 'ultrakill.wav    ';
				$this->KILL_07_LINK = $rep_sounds . 'wickedsick.wav   ';
				$this->KILL_08_LINK = $rep_sounds . 'monsterkill.wav  ';
				$this->KILL_09_LINK = $rep_sounds . 'unstoppable.wav  ';
				$this->KILL_10_LINK = $rep_sounds . 'godlike.wav      ';
				$this->KILL_11_LINK = $rep_sounds . 'holyshit.wav     ';
				$this->KILL_12_LINK = $rep_sounds . 'LudicrousKill.wav';
				$this->KILL_13_LINK = '';
				$this->KILL_14_LINK = '';
				$this->KILL_15_LINK = '';
				$this->KILL_16_LINK = '';
				$this->KILL_17_LINK = '';
				$this->KILL_18_LINK = '';
				$this->KILL_19_LINK = '';
				$this->KILL_20_LINK = '';
				$this->KILL_21_LINK = '';
				$this->KILL_22_LINK = '';
				$this->KILL_23_LINK = '';
				$this->KILL_24_LINK = '';
				$this->KILL_25_LINK = '';
				$this->KILL_26_LINK = '';
				$this->KILL_27_LINK = '';
				$this->KILL_28_LINK = '';
				$this->KILL_29_LINK = '';
				$this->KILL_30_LINK = '';
				break;
		}
		*/
		
		// SERVER specific
		/*
		switch ($serverName) {
			// SpeedBall default config
			case 'mm2monde.fr':
				//$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'none');
				break;
			case 'TO DO':
				// BLUE/RED TEAM SCORES : allow
				//$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'none');
				break;
			default:
				// BLUE/RED TEAM SCORES : allow
				//$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'all');
				break;
		}
		*/
		
		
		// SETTINGS TO DELETE (when their value have changed)
		$this->maniaControl->settingManager->deleteSetting($this, '00b1. MUSIC enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08c. HIT enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08d. HIT sound');
		$this->maniaControl->settingManager->deleteSetting($this, '000. For any bug/suggestion : $lhttps://forum.maniaplanet.com/viewtopic.php?f=479&t=30285&p=238505#p238505');
		$this->maniaControl->settingManager->deleteSetting($this, '00j. Reset killstreak on new round');
		
		$this->maniaControl->settingManager->deleteSetting($this, '05a. TEAM_SCORES enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '06a. BACKSPACE enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '07a. OFF-ZONE enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08a. HIT_LASER enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08c. HIT_ROCKET enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08e. HIT_NUCLEUS enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08g. HIT_ARROW enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '09a. TEAMKILLER enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '100. KILLSTREAK (consecutive kills without dying) enabled ?');
		
		$this->maniaControl->settingManager->deleteSetting($this, '00f. MESSAGES : timeout');
		
		// V1.07
		// Renamed
		$this->maniaControl->settingManager->deleteSetting($this, '000. For any bug/suggestion/...');
		$this->maniaControl->settingManager->deleteSetting($this, '001. XML config file');
		$this->maniaControl->settingManager->deleteSetting($this, '001. XML configuration file');
		$this->maniaControl->settingManager->deleteSetting($this, '001. CONFIG_XML : XML configuration file');
		$this->maniaControl->settingManager->deleteSetting($this, '00a. DEBUG ');
		$this->maniaControl->settingManager->deleteSetting($this, '00b. VOLUME');
		$this->maniaControl->settingManager->deleteSetting($this, '00b00. ICON MENU : X position');
		$this->maniaControl->settingManager->deleteSetting($this, '00b01. ICON MENU : Y position');
		$this->maniaControl->settingManager->deleteSetting($this, '00b02. ICON MENU : width');
		$this->maniaControl->settingManager->deleteSetting($this, '00b03. ICON MENU : height');
		$this->maniaControl->settingManager->deleteSetting($this, '00b1. MUSIC enabled on new map ?');
		$this->maniaControl->settingManager->deleteSetting($this, '00b2. MUSIC VOLUME');
		$this->maniaControl->settingManager->deleteSetting($this, '00b4. MUSIC delay in ms on new map');
		$this->maniaControl->settingManager->deleteSetting($this, '00c0. MESSAGES enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '00c. MESSAGES : X position');
		$this->maniaControl->settingManager->deleteSetting($this, '00d. MESSAGES : Y position');
		$this->maniaControl->settingManager->deleteSetting($this, '00e. MESSAGES : scale');
		$this->maniaControl->settingManager->deleteSetting($this, '00f. MESSAGES : duration');
		$this->maniaControl->settingManager->deleteSetting($this, '00f0. RANKING enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '00f. RANKING : X position');
		$this->maniaControl->settingManager->deleteSetting($this, '00g. RANKING : Y position');
		$this->maniaControl->settingManager->deleteSetting($this, '00h. RANKING : scale');
		$this->maniaControl->settingManager->deleteSetting($this, '00i. IGNORE_LIST');
		$this->maniaControl->settingManager->deleteSetting($this, '01a. PLAYER_CONNECTED enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '02a. PLAYER_DISCONNECTED enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '03a. BEGIN_MAP enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '03c. BEGIN_MAP delay in ms');
		$this->maniaControl->settingManager->deleteSetting($this, '04a. BEGIN_PLAYING enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '04c. BEGIN_PLAYING delay in ms');
		$this->maniaControl->settingManager->deleteSetting($this, '05a. TEAM_SCORES enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '05d. TEAM_SCORES delay in ms');
		$this->maniaControl->settingManager->deleteSetting($this, '06a. BACKSPACE enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '07a. OFF-ZONE enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08a. HIT_LASER enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08c. HIT_ROCKET enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08e. HIT_NUCLEUS enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08g. HIT_ARROW enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '09a. TEAMKILLER enabled to [none/player/team/all] ?');
		
		$this->maniaControl->settingManager->deleteSetting($this, '009. PLAYER_CONNECTED enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '010. PLAYER_DISCONNECTED enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '011a. BEGIN_MAP enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '012a. BEGIN_PLAYING enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '012d. BEGIN_PLAYING delay in ms');
		$this->maniaControl->settingManager->deleteSetting($this, '013a. TEAM_SCORES enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '014. BACKSPACE enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '015. OFF-ZONE enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '016a. HIT_LASER enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '016b. HIT_ROCKET enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '016c. HIT_NUCLEUS enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '016d. HIT_ARROW enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '017. TEAMKILLER enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '100. KILLSTREAK enabled to [none/player/team/all] ?');
		$this->maniaControl->settingManager->deleteSetting($this, '100.00. Reset KILLSTREAK on new round ?');
		
		// Deleted (moved to XML file)
		$this->maniaControl->settingManager->deleteSetting($this, '00b3. MUSIC link'               );
		$this->maniaControl->settingManager->deleteSetting($this, '01b. PLAYER_CONNECTED sound'    );
		$this->maniaControl->settingManager->deleteSetting($this, '02b. PLAYER_DISCONNECTED sound' );
		$this->maniaControl->settingManager->deleteSetting($this, '03b. BEGIN_MAP sound'           );
		$this->maniaControl->settingManager->deleteSetting($this, '04b. BEGIN_PLAYING sound'       );
		$this->maniaControl->settingManager->deleteSetting($this, '05b. BLUE_TEAM_SCORES sound'    );
		$this->maniaControl->settingManager->deleteSetting($this, '05c. RED_TEAM_SCORES sound'     );
		$this->maniaControl->settingManager->deleteSetting($this, '06b. BACKSPACE sound'           );
		$this->maniaControl->settingManager->deleteSetting($this, '07b. OFF-ZONE sound'            );
		$this->maniaControl->settingManager->deleteSetting($this, '08b. HIT_LASER sound'           );
		$this->maniaControl->settingManager->deleteSetting($this, '08d. HIT_ROCKET sound'          );
		$this->maniaControl->settingManager->deleteSetting($this, '08f. HIT_NUCLEUS sound'         );
		$this->maniaControl->settingManager->deleteSetting($this, '08h. HIT_ARROW sound'           );
		$this->maniaControl->settingManager->deleteSetting($this, '09b. TEAMKILLER sound'          );
		$this->maniaControl->settingManager->deleteSetting($this, '100.01. Sound for kill 01'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.02. Sound for kill 02'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.03. Sound for kill 03'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.04. Sound for kill 04'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.05. Sound for kill 05'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.06. Sound for kill 06'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.07. Sound for kill 07'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.08. Sound for kill 08'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.09. Sound for kill 09'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.10. Sound for kill 10'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.11. Sound for kill 11'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.12. Sound for kill 12'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.13. Sound for kill 13'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.14. Sound for kill 14'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.15. Sound for kill 15'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.16. Sound for kill 16'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.17. Sound for kill 17'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.18. Sound for kill 18'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.19. Sound for kill 19'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.20. Sound for kill 20'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.21. Sound for kill 21'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.22. Sound for kill 22'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.23. Sound for kill 23'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.24. Sound for kill 24'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.25. Sound for kill 25'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.26. Sound for kill 26'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.27. Sound for kill 27'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.28. Sound for kill 28'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.29. Sound for kill 29'      );
		$this->maniaControl->settingManager->deleteSetting($this, '100.30. Sound for kill 30'      );
		
		$this->maniaControl->settingManager->deleteSetting($this, '003. VOLUME'      );
		
		// ICONS for MENU
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MENU				, self::SEM_ICON_MENU_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MENU_MOVER			, self::SEM_ICON_MENU_MOVER_PATH );
		
		// ICONS for SOUNDS
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_SOUND_ID, $this, 'enable_disable_sounds');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_ON			, self::SEM_ICON_SOUND_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_OFF			, self::SEM_ICON_SOUND_OFF_PATH      );
		
		// ICONS for MESSAGES
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_MESSAGE_ID, $this, 'enable_disable_messages');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MESSAGE_ON			, self::SEM_ICON_MESSAGE_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MESSAGE_OFF			, self::SEM_ICON_MESSAGE_OFF_PATH      );
		
		// ICONS for RANKING
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_RANKING_ID, $this, 'enable_disable_ranking');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_RANKING_ON			, self::SEM_ICON_RANKING_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_RANKING_OFF			, self::SEM_ICON_RANKING_OFF_PATH      );
		
		// ICONS for ENABLE / DISABLE MUSIC
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_MUSIC_ID, $this, 'enable_disable_music');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_ON			, self::SEM_ICON_MUSIC_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_OFF			, self::SEM_ICON_MUSIC_OFF_PATH      );
		
		// ICONS for PLAY / STOP MUSIC
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_PLAY_ID, $this, 'play_stop_music');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_PLAY			, self::SEM_ICON_MUSIC_PLAY_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_STOP			, self::SEM_ICON_MUSIC_STOP_PATH      );
		
		// ICONS for SOUND ON DEMAND
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_SOD_MENU_ID, $this, 'open_sound_on_demand');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOD			, self::SEM_ICON_SOD_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOD_MOVER	, self::SEM_ICON_SOD_MOVER_PATH );
		
		// ICONS for ENABLE / DISABLE SOUND ON DEMAND
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_SOD_ON_OFF_ID, $this, 'enable_disable_sound_on_demand');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOD_ON			, self::SEM_ICON_SOD_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOD_OFF			, self::SEM_ICON_SOD_OFF_PATH      );
		
		
		// LOAD CONFIGURATION FROM CONFIG_XML / CONFIG_XML_DEFAULT FILE
		// Check if XML config files exists
		if (file_exists($this->CONFIG_XML)) {
			$xmlFile = $this->CONFIG_XML;
		}
		else if (file_exists($this->CONFIG_XML_DEFAULT)) {
			$xmlFile = $this->CONFIG_XML_DEFAULT;
			//$this->maniaControl->chat->sendInformation('[Sound Events Manager] ' . $this->CONFIG_XML . ' is not set. Using ' . $this->CONFIG_XML_DEFAULT . '.', null, false);
			$this->sendInformation($this->CONFIG_XML . ' is not set. Using ' . $this->CONFIG_XML_DEFAULT . '.', null);
		}
		else {
			$xmlFile = '';
			//$this->maniaControl->chat->sendInformation('[Sound Events Manager] ' . $this->CONFIG_XML . ' and ' . $this->CONFIG_XML_DEFAULT . ' are not set. SoundEventsManager disabled.', null, false);
			$this->sendInformation($this->CONFIG_XML . ' and ' . $this->CONFIG_XML_DEFAULT . ' are not set. SoundEventsManager disabled.', null);
			$this->unload();
			return;
		}
		// Get parameters from XML config file
		if ($xml = simplexml_load_string(file_get_contents($xmlFile, FILE_USE_INCLUDE_PATH))) {
			foreach($xml->sound as $sound) {
				// Get tags value inside <sound></sound>
				$name = $sound->name;
				$description = $sound->description;
				$url = $sound->url;
				$blockothersounds = $sound->blockothersounds;
				$duration = $sound->duration;
				
				// Check non-optionnal tags
				if (empty($name) || empty($url)) {
					if (!empty($name)) {
						//$this->maniaControl->chat->sendInformation('[Sound Events Manager] Tag <url> is missing for section \'' . $name . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null, false);
						$this->sendInformation('Tag <url> is missing for section \'' . $name . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null);
					}
					else if (!empty($description)) {
						//$this->maniaControl->chat->sendInformation('[Sound Events Manager] Tag <name> or <url> is missing for description \'' . $description . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null, false);
						$this->sendInformation('Tag <name> or <url> is missing for description \'' . $description . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null);
					}
					else {
						//$this->maniaControl->chat->sendInformation('[Sound Events Manager] One tag <name> or <url> is missing. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null, false);
						$this->sendInformation('One tag <name> or <url> is missing. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null);
					}
					$this->unload();
					return;
				}
				// Force optionnal tags to default values if their value is wrong
				if ($blockothersounds != 'true') { $blockothersounds = 'false'; }
				if (!is_numeric($duration)) { $duration = 0; }
				
				// Store values into the good variable
				// NB : using switch() instead of dynamical variables to store only desired <tag>
				switch($name) {
					case 'MUSIC' :
						$this->MUSIC_LINK = $url;
						$this->MUSIC_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->MUSIC_DURATION = $duration;
						break;
					case 'PLAYER_CONNECTED' :
						$this->PLAYER_CONNECTED_LINK = $url;
						$this->PLAYER_CONNECTED_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->PLAYER_CONNECTED_DURATION = $duration;
						break;
					case 'PLAYER_DISCONNECTED' :
						$this->PLAYER_DISCONNECTED_LINK = $url;
						$this->PLAYER_DISCONNECTED_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->PLAYER_DISCONNECTED_DURATION = $duration;
						break;
					case 'BEGIN_MAP' :
						$this->BEGIN_MAP_LINK = $url;
						$this->BEGIN_MAP_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->BEGIN_MAP_DURATION = $duration;
						break;
					case 'BEGIN_PLAYING' :
						$this->BEGIN_PLAYING_LINK = $url;
						$this->BEGIN_PLAYING_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->BEGIN_PLAYING_DURATION = $duration;
						break;
					case 'BLUE_TEAM_SCORES' :
						$this->BLUE_TEAM_SCORES_LINK = $url;
						$this->BLUE_TEAM_SCORES_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->BLUE_TEAM_SCORES_DURATION = $duration;
						break;
					case 'RED_TEAM_SCORES' :
						$this->RED_TEAM_SCORES_LINK = $url;
						$this->RED_TEAM_SCORES_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->RED_TEAM_SCORES_DURATION = $duration;
						break;
					case 'BACKSPACE' :
						$this->BACKSPACE_LINK = $url;
						$this->BACKSPACE_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->BACKSPACE_DURATION = $duration;
						break;
					case 'OFFZONE' :
						$this->OFFZONE_LINK = $url;
						$this->OFFZONE_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->OFFZONE_DURATION = $duration;
						break;
					case 'HIT_LASER' :
						$this->HIT_LASER_LINK = $url;
						$this->HIT_LASER_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->HIT_LASER_DURATION = $duration;
						break;
					case 'HIT_ROCKET' :
						$this->HIT_ROCKET_LINK = $url;
						$this->HIT_ROCKET_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->HIT_ROCKET_DURATION = $duration;
						break;
					case 'HIT_NUCLEUS' :
						$this->HIT_NUCLEUS_LINK = $url;
						$this->HIT_NUCLEUS_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->HIT_NUCLEUS_DURATION = $duration;
						break;
					case 'HIT_ARROW' :
						$this->HIT_ARROW_LINK = $url;
						$this->HIT_ARROW_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->HIT_ARROW_DURATION = $duration;
						break;
					case 'TEAMKILLER' :
						$this->TEAMKILLER_LINK = $url;
						$this->TEAMKILLER_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->TEAMKILLER_DURATION = $duration;
						break;
					case 'KILL_01' :
						$this->KILL_01_LINK = $url;
						$this->KILL_01_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_01_DURATION = $duration;
						break;
					case 'KILL_02' :
						$this->KILL_02_LINK = $url;
						$this->KILL_02_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_02_DURATION = $duration;
						break;
					case 'KILL_03' :
						$this->KILL_03_LINK = $url;
						$this->KILL_03_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_03_DURATION = $duration;
						break;
					case 'KILL_04' :
						$this->KILL_04_LINK = $url;
						$this->KILL_04_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_04_DURATION = $duration;
						break;
					case 'KILL_05' :
						$this->KILL_05_LINK = $url;
						$this->KILL_05_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_05_DURATION = $duration;
						break;
					case 'KILL_06' :
						$this->KILL_06_LINK = $url;
						$this->KILL_06_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_06_DURATION = $duration;
						break;
					case 'KILL_07' :
						$this->KILL_07_LINK = $url;
						$this->KILL_07_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_07_DURATION = $duration;
						break;
					case 'KILL_08' :
						$this->KILL_08_LINK = $url;
						$this->KILL_08_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_08_DURATION = $duration;
						break;
					case 'KILL_09' :
						$this->KILL_09_LINK = $url;
						$this->KILL_09_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_09_DURATION = $duration;
						break;
					case 'KILL_10' :
						$this->KILL_10_LINK = $url;
						$this->KILL_10_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_10_DURATION = $duration;
						break;
					case 'KILL_11' :
						$this->KILL_11_LINK = $url;
						$this->KILL_11_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_11_DURATION = $duration;
						break;
					case 'KILL_12' :
						$this->KILL_12_LINK = $url;
						$this->KILL_12_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_12_DURATION = $duration;
						break;
					case 'KILL_13' :
						$this->KILL_13_LINK = $url;
						$this->KILL_13_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_13_DURATION = $duration;
						break;
					case 'KILL_14' :
						$this->KILL_14_LINK = $url;
						$this->KILL_14_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_14_DURATION = $duration;
						break;
					case 'KILL_15' :
						$this->KILL_15_LINK = $url;
						$this->KILL_15_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_15_DURATION = $duration;
						break;
					case 'KILL_16' :
						$this->KILL_16_LINK = $url;
						$this->KILL_16_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_16_DURATION = $duration;
						break;
					case 'KILL_17' :
						$this->KILL_17_LINK = $url;
						$this->KILL_17_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_17_DURATION = $duration;
						break;
					case 'KILL_18' :
						$this->KILL_18_LINK = $url;
						$this->KILL_18_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_18_DURATION = $duration;
						break;
					case 'KILL_19' :
						$this->KILL_19_LINK = $url;
						$this->KILL_19_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_19_DURATION = $duration;
						break;
					case 'KILL_20' :
						$this->KILL_20_LINK = $url;
						$this->KILL_20_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_20_DURATION = $duration;
						break;
					case 'KILL_21' :
						$this->KILL_21_LINK = $url;
						$this->KILL_21_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_21_DURATION = $duration;
						break;
					case 'KILL_22' :
						$this->KILL_22_LINK = $url;
						$this->KILL_22_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_22_DURATION = $duration;
						break;
					case 'KILL_23' :
						$this->KILL_23_LINK = $url;
						$this->KILL_23_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_23_DURATION = $duration;
						break;
					case 'KILL_24' :
						$this->KILL_24_LINK = $url;
						$this->KILL_24_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_24_DURATION = $duration;
						break;
					case 'KILL_25' :
						$this->KILL_25_LINK = $url;
						$this->KILL_25_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_25_DURATION = $duration;
						break;
					case 'KILL_26' :
						$this->KILL_26_LINK = $url;
						$this->KILL_26_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_26_DURATION = $duration;
						break;
					case 'KILL_27' :
						$this->KILL_27_LINK = $url;
						$this->KILL_27_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_27_DURATION = $duration;
						break;
					case 'KILL_28' :
						$this->KILL_28_LINK = $url;
						$this->KILL_28_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_28_DURATION = $duration;
						break;
					case 'KILL_29' :
						$this->KILL_29_LINK = $url;
						$this->KILL_29_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_29_DURATION = $duration;
						break;
					case 'KILL_30' :
						$this->KILL_30_LINK = $url;
						$this->KILL_30_BLOCKOTHERSOUNDS = $blockothersounds;
						$this->KILL_30_DURATION = $duration;
						break;
					default :
						//$this->maniaControl->chat->sendInformation('[Sound Events Manager] Tag "' . $name . '" unknown. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null, false);
						$this->sendInformation('Tag "' . $name . '" unknown. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null);
						$this->unload();
						break;
				}
			}
		}
		// Error while parsing CONFIG_XML / CONFIG_XML_DEFAULT file
		else {
			libxml_clear_errors();
			//$this->maniaControl->chat->sendInformation('[Sound Events Manager] Error while parsing ' . $xmlFile . '. Please check your file. SoundEventsManager disabled.', null, false);
			$this->sendInformation('Error while parsing ' . $xmlFile . '. Please check your file. SoundEventsManager disabled.', null);
			$this->unload();
			return;
		}
		
		// LOAD CONFIGURATION FROM SOUND_ON_DEMAND_XML / SOUND_ON_DEMAND_XML_DEFAULT FILE
		// Check if XML SoD config files exists
		if (file_exists($this->SOUND_ON_DEMAND_XML)) {
			$xmlFile = $this->SOUND_ON_DEMAND_XML;
		}
		else if (file_exists($this->SOUND_ON_DEMAND_XML_DEFAULT)) {
			$xmlFile = $this->SOUND_ON_DEMAND_XML_DEFAULT;
			//$this->maniaControl->chat->sendInformation('[Sound Events Manager] ' . $this->SOUND_ON_DEMAND_XML . ' is not set. Using ' . $this->SOUND_ON_DEMAND_XML_DEFAULT . '.', null, false);
			$this->sendInformation($this->SOUND_ON_DEMAND_XML . ' is not set. Using ' . $this->SOUND_ON_DEMAND_XML_DEFAULT . '.', null);
		}
		else {
			$xmlFile = '';
			//$this->maniaControl->chat->sendInformation('[Sound Events Manager] ' . $this->SOUND_ON_DEMAND_XML . ' and ' . $this->SOUND_ON_DEMAND_XML_DEFAULT . ' are not set. No SOUND ON DEMAND will be available.', null, false);
			$this->sendInformation($this->SOUND_ON_DEMAND_XML . ' and ' . $this->SOUND_ON_DEMAND_XML_DEFAULT . ' are not set. No SOUND ON DEMAND will be available.', null);
		}
		// Get parameters from XML SoD file
		$CptSoD = 0;
		if ($xmlFile != '') {
			if ($xml = simplexml_load_string(file_get_contents($xmlFile, FILE_USE_INCLUDE_PATH))) {
				foreach($xml->sound as $sound) {
					// Get tags value inside <sound></sound>
					$name = $sound->name;
					$description = $sound->description;
					$url = $sound->url;
					$blockothersounds = $sound->blockothersounds;
					$duration = $sound->duration;
					
					// Check non-optionnal tags
					if (empty($name) || empty($url)) {
						if (!empty($name)) {
							//$this->maniaControl->chat->sendInformation('[Sound Events Manager] Tag <url> is missing for section \'' . $name . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null, false);
							$this->sendInformation('Tag <url> is missing for section \'' . $name . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null);
						}
						else if (!empty($description)) {
							//$this->maniaControl->chat->sendInformation('[Sound Events Manager] Tag <name> or <url> is missing for description \'' . $description . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null, false);
							$this->sendInformation('Tag <name> or <url> is missing for description \'' . $description . '\'. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null);
						}
						else {
							//$this->maniaControl->chat->sendInformation('[Sound Events Manager] One tag <name> or <url> is missing. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null, false);
							$this->sendInformation('One tag <name> or <url> is missing. Please check ' . $xmlFile . '. SoundEventsManager disabled.', null);
						}
						$this->unload();
						return;
					}
					
					// Force optionnal tags to default values if their value is wrong
					if ($blockothersounds != 'true') { $blockothersounds = 'false'; }
					$duration = intval($duration); // Force to 0 if not numerical
					//$duration_int = intval($duration);
					//if (!is_numeric($duration)) { $duration = 0; }
					
				
					// Store parameters into global variables
					$CptSoD++;
					$CptSoD_2digits = sprintf("%02u",$CptSoD);
					$this->{"SOUND_" . $CptSoD_2digits . "_NAME"} = substr($name, 0, 20);	// NB : SOUND_*_NAME can't have more than 20 character to fit in display
					$this->{"SOUND_" . $CptSoD_2digits . "_DESCRIPTION"} = $description;
					$this->{"SOUND_" . $CptSoD_2digits . "_LINK"} = $url;
					$this->{"SOUND_" . $CptSoD_2digits . "_BLOCKOTHERSOUNDS"} = $blockothersounds;
					$this->{"SOUND_" . $CptSoD_2digits . "_DURATION"} = $duration;
					// DEBUG
					$this->toLog("SOUND_ON_DEMAND_XML values n°$CptSoD_2digits : $name / $description / $url / $blockothersounds / $duration");
				}
			}
			// Error while parsing SOUND_ON_DEMAND_XML / SOUND_ON_DEMAND_XML_DEFAULT files
			else {
				libxml_clear_errors();
				//$this->maniaControl->chat->sendInformation('[Sound Events Manager] Error while parsing ' . $xmlFile . '. Please check your file. SoundEventsManager disabled.', null, false);
				$this->sendInformation('Error while parsing ' . $xmlFile . '. Please check your file. SoundEventsManager disabled.', null);
				$this->unload();
				return;
			}
		}
		
		// Music to play ?
		$this->musicToPlay();
		
		// Add icons to menu for each player
		$this->addMenuIconToAll(false);
		
		// Preload all sounds for all players
		$this->preloadAllSoundsToAll();
    }
	
	
	
	/**
	 * Force everybody to preload all sounds.
	 *
	 */
	public function preloadAllSoundsToAll() {
		// Display the SEM icons only for non-ignored players
		foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			$login = $player->login;
			if (!$this->isIgnored($login)) {
				$this->preloadAllSounds($login);
			}
		}
	}
	
	
	
	/**
	 * Make $login to download all sounds. He won't download them twice, as they will be stored into the cache.
	 */
	public function preloadAllSounds($login) {
		// Don't download sounds if player is IGNORED
		if ($this->isIgnored($login)) { return ; }
		
		// All sounds which can be played.
		$allSoundsArray = array(
			$this->MUSIC_LINK,
			$this->PLAYER_CONNECTED_LINK,
			$this->PLAYER_DISCONNECTED_LINK,
			$this->BEGIN_MAP_LINK,
			$this->BEGIN_PLAYING_LINK,
			$this->BLUE_TEAM_SCORES_LINK,
			$this->RED_TEAM_SCORES_LINK,
			$this->BACKSPACE_LINK,
			$this->OFFZONE_LINK,
			$this->HIT_LASER_LINK,
			$this->HIT_ROCKET_LINK,
			$this->HIT_NUCLEUS_LINK,
			$this->HIT_ARROW_LINK,
			$this->TEAMKILLER_LINK,
			// V1.08 $this->maniaControl->getSettingManager()->getSettingValue($this, self::ONNEARMISS_LINK         ),
			// V1.08 $this->maniaControl->getSettingManager()->getSettingValue($this, self::BEGINWARMUP_LINK        ),
			// V1.08 $this->maniaControl->getSettingManager()->getSettingValue($this, self::ENDWARMUP_LINK          ),
			// V1.08 $this->maniaControl->getSettingManager()->getSettingValue($this, self::ENDPLAYING_LINK         ),
			// V1.08 $this->maniaControl->getSettingManager()->getSettingValue($this, self::ENDMAP_LINK             ),
			$this->KILL_01_LINK,
			$this->KILL_02_LINK,
			$this->KILL_03_LINK,
			$this->KILL_04_LINK,
			$this->KILL_05_LINK,
			$this->KILL_06_LINK,
			$this->KILL_07_LINK,
			$this->KILL_08_LINK,
			$this->KILL_09_LINK,
			$this->KILL_10_LINK,
			$this->KILL_11_LINK,
			$this->KILL_12_LINK,
			$this->KILL_13_LINK,
			$this->KILL_14_LINK,
			$this->KILL_15_LINK,
			$this->KILL_16_LINK,
			$this->KILL_17_LINK,
			$this->KILL_18_LINK,
			$this->KILL_19_LINK,
			$this->KILL_20_LINK,
			$this->KILL_21_LINK,
			$this->KILL_22_LINK,
			$this->KILL_23_LINK,
			$this->KILL_24_LINK,
			$this->KILL_25_LINK,
			$this->KILL_26_LINK,
			$this->KILL_27_LINK,
			$this->KILL_28_LINK,
			$this->KILL_29_LINK,
			$this->KILL_30_LINK,
			$this->SOUND_01_LINK,
			$this->SOUND_02_LINK,
			$this->SOUND_03_LINK,
			$this->SOUND_04_LINK,
			$this->SOUND_05_LINK,
			$this->SOUND_06_LINK,
			$this->SOUND_07_LINK,
			$this->SOUND_08_LINK,
			$this->SOUND_09_LINK,
			$this->SOUND_10_LINK,
			$this->SOUND_11_LINK,
		 	$this->SOUND_12_LINK,
			$this->SOUND_13_LINK,
			$this->SOUND_14_LINK,
			$this->SOUND_15_LINK,
			$this->SOUND_16_LINK,
			$this->SOUND_17_LINK,
			$this->SOUND_18_LINK,
			$this->SOUND_19_LINK,
			$this->SOUND_20_LINK,
			$this->SOUND_21_LINK,
			$this->SOUND_22_LINK,
			$this->SOUND_23_LINK,
			$this->SOUND_24_LINK,
			$this->SOUND_25_LINK,
			$this->SOUND_26_LINK,
			$this->SOUND_27_LINK,
			$this->SOUND_28_LINK,
			$this->SOUND_29_LINK,
			$this->SOUND_30_LINK,
			$this->SOUND_31_LINK,
			$this->SOUND_32_LINK,
			$this->SOUND_33_LINK,
			$this->SOUND_34_LINK,
			$this->SOUND_35_LINK,
			$this->SOUND_36_LINK,
			$this->SOUND_37_LINK,
			$this->SOUND_38_LINK,
			$this->SOUND_39_LINK,
			$this->SOUND_40_LINK,
			$this->SOUND_41_LINK,
		 	$this->SOUND_42_LINK,
			$this->SOUND_43_LINK,
			$this->SOUND_44_LINK,
			$this->SOUND_45_LINK,
			$this->SOUND_46_LINK,
			$this->SOUND_47_LINK,
			$this->SOUND_48_LINK,
			$this->SOUND_49_LINK,
			$this->SOUND_50_LINK,
			$this->SOUND_51_LINK,
			$this->SOUND_52_LINK,
			$this->SOUND_53_LINK,
			$this->SOUND_54_LINK,
			$this->SOUND_55_LINK,
			$this->SOUND_56_LINK,
			$this->SOUND_57_LINK,
			$this->SOUND_58_LINK,
			$this->SOUND_59_LINK,
			$this->SOUND_60_LINK,
			$this->SOUND_61_LINK,
			$this->SOUND_62_LINK,
			$this->SOUND_63_LINK,
			$this->SOUND_64_LINK,
			$this->SOUND_65_LINK,
			$this->SOUND_66_LINK,
			$this->SOUND_67_LINK,
			$this->SOUND_68_LINK,
			$this->SOUND_69_LINK,
			$this->SOUND_70_LINK,
			$this->SOUND_71_LINK,
			$this->SOUND_72_LINK,
			$this->SOUND_73_LINK,
			$this->SOUND_74_LINK,
			$this->SOUND_75_LINK,
			$this->SOUND_76_LINK,
			$this->SOUND_77_LINK,
			$this->SOUND_78_LINK,
			$this->SOUND_79_LINK,
			$this->SOUND_80_LINK,
			$this->SOUND_81_LINK,
		 	$this->SOUND_82_LINK,
			$this->SOUND_83_LINK,
			$this->SOUND_84_LINK,
			$this->SOUND_85_LINK,
			$this->SOUND_86_LINK,
			$this->SOUND_87_LINK,
			$this->SOUND_88_LINK,
			$this->SOUND_89_LINK,
			$this->SOUND_90_LINK,
			$this->SOUND_91_LINK,
			$this->SOUND_92_LINK,
			$this->SOUND_93_LINK,
			$this->SOUND_94_LINK,
			$this->SOUND_95_LINK,
			$this->SOUND_96_LINK,
			$this->SOUND_97_LINK,
			$this->SOUND_98_LINK,
			$this->SOUND_99_LINK
		);
		// Removes all NULL, FALSE and Empty Strings but leaves 0 (zero) values
		$allSoundsArray = array_filter($allSoundsArray, 'strlen');
		
		// Create an ManiaLink for the FML
		$maniaLink = new \FML\ManiaLink(self::ML_SOUND_ID);
		
		// Create an audio element from FML and add it to the manialink
		foreach ($allSoundsArray as $link) {
			$audio = new \FML\Controls\Audio();
			$maniaLink->add($audio);
			$audio->setVolume(0);		//							  from FML/Types/Playable.php
			$audio->setPlay(0);			// Don't play automatically	- from FML/Types/Playable.php
			$audio->setLooping(0);		// Play only once			- from FML/Types/Playable.php
			$audio->setVisible(0);		// Popup not visible		- from FML/Controls/Control.php
			$audio->setData($link);
		}
		
		// Message "preloading in progress"
		//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Preloading all sounds and music (if not already downloaded)...', $login, false);
		$PRELOAD_MESSAGE_ACTIVATED = $this->maniaControl->getSettingManager()->getSettingValue($this, self::PRELOAD_MESSAGE_ACTIVATED);
		if ($PRELOAD_MESSAGE_ACTIVATED) { $this->sendInformation('Preloading all sounds and music (if not already downloaded)...', $login); }
		
		// Send ManiaLink with a duration of 10ms (just to download them, not to hear them)
		$this->maniaControl->manialinkManager->sendManialink($maniaLink, $login, 10);
	}
	
	
	
	/**
	 * Display the Menu Icons for SEM to everybody
	 *
	 */
	public function addMenuIconToAll($visible) {
		// Display the SEM icons only for non-ignored players
		foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			if (!$this->isIgnored($player->login)) {
				$this->addMenuIcon($player, $visible);
			}
		}
	}
	
	/**
	 * Display the SEM Menu Icons for $player
	 *
	 * Mainly copy/paste from displayDonateWidget() in DonationPlugin.php
	 *
	 */
	public function addMenuIcon($player, $visible) {
		$posX              = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SEM_ICON_MENU_POS_X);
		$posY              = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SEM_ICON_MENU_POS_Y);
		$width             = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SEM_ICON_MENU_WIDTH);
		$height            = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SEM_ICON_MENU_HEIGHT);
		$scale             = 0.8;
		$shootManiaOffset  = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultIconOffsetSM();
		$quadStyle         = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultQuadStyle();
		$quadSubstyle      = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultQuadSubstyle();
		$itemMarginFactorX = 1.3;
		$itemMarginFactorY = 1.2;
		// $itemNumber        = 6;
		// TO DO : replace with $player->isAdmin() ?
		if ($this->maniaControl->getAuthenticationManager()->checkRight($player, AuthenticationManager::AUTH_LEVEL_ADMIN)) { $itemNumber = 7; }
		else { $itemNumber = 5; }
		$itemPlace = -1;
		
		// PLAY/STOP MUSIC button displayed only if a link is set
		if ($this->MUSIC_LINK == '') { $itemNumber--; }
		
		//If game is shootmania lower the icons position by 20
		if ($this->maniaControl->getMapManager()->getCurrentMap()->getGame() == 'sm') {
			$posY -= $shootManiaOffset;
		}
		
		$itemSize = $width;
		
		// Setting player's variables
		$playerLogin = $player->login;
		
		$maniaLink = new ManiaLink(self::ML_ICON_MENU_ID);
		
		// SEM MENU ICON FRAME
		$frame = new Frame();
		$maniaLink->add($frame);
		$frame->setPosition($posX, $posY);

		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		$backgroundQuad->setSize($width * $itemMarginFactorX, $height * $itemMarginFactorY);
		$backgroundQuad->setStyles($quadStyle, $quadSubstyle);

		$iconFrame = new Frame();
		$frame->add($iconFrame);
		
		// "SEM" ICON
		$iconFrame->setSize($itemSize, $itemSize);
		$itemQuad = new Quad();
		$itemQuad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MENU));
		$itemQuad->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MENU_MOVER));
		$itemQuad->setSize($itemSize, $itemSize);
		$iconFrame->add($itemQuad);
		
		// POPUP MENU
		$popoutFrame = new Frame();
		$maniaLink->add($popoutFrame);
		$popoutFrame->setPosition($posX - $itemSize * 0.5, $posY);
		$popoutFrame->setHAlign($popoutFrame::RIGHT);
		$popoutFrame->setSize($itemSize * 1.2 * $itemNumber, $itemSize * $itemMarginFactorY);
		$popoutFrame->setVisible($visible);
		
		$quad = new Quad();
		$popoutFrame->add($quad);
		$quad->setHAlign($quad::RIGHT);
		$quad->setStyles($quadStyle, $quadSubstyle);
		$quad->setSize($itemSize * 1.2 * $itemNumber, $itemSize * $itemMarginFactorY);
		//$popoutFrame->add($quad);
		$itemQuad->addToggleFeature($popoutFrame);
		
		// DESCRIPTION LABEL
		$descriptionFrame = new Frame();
		$maniaLink->add($descriptionFrame);
		$descriptionFrame->setPosition($posX - $itemSize * $itemNumber * 1.35, $posY);
		$descriptionFrame->setHAlign($descriptionFrame::RIGHT);

		$descriptionLabel = new Label();
		$descriptionFrame->add($descriptionLabel);
		$descriptionLabel->setAlign($descriptionLabel::RIGHT, $descriptionLabel::TOP);
		$descriptionLabel->setSize(30, 2);
		$descriptionLabel->setTextSize(1);
		$descriptionLabel->setVisible(true);
		$descriptionLabel->setTextColor('fff');
		
		$itemQuad->addTooltipLabelFeature($descriptionLabel, "Sound Events Manager");
		
		// SOUNDS ICON
		if ($this->hasDisabledSounds($playerLogin)) {
			// Sounds are OFF
			$itemQuadSound = new Quad();
			$popoutFrame->add($itemQuadSound);
			$itemQuadSound->setPosition($itemSize * $itemPlace, 0);
			$itemQuadSound->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadSound->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_OFF));
			$itemQuadSound->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_ON));
			$itemQuadSound->setAction(self::ML_ICON_SOUND_ID);
			$itemQuadSound->addTooltipLabelFeature($descriptionLabel, "Enable sounds");
		} else {
			// Sounds are ON
			$itemQuadSound = new Quad();
			$popoutFrame->add($itemQuadSound);
			$itemQuadSound->setPosition($itemSize * $itemPlace, 0);
			$itemQuadSound->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadSound->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_ON));
			$itemQuadSound->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_OFF));
			$itemQuadSound->setAction(self::ML_ICON_SOUND_ID);
			$itemQuadSound->addTooltipLabelFeature($descriptionLabel, "Disable sounds");
		}
		$itemPlace--;
		
		// MESSAGES ICON
		if ($this->hasDisabledMessages($playerLogin)) {
			// Message are OFF
			$itemQuadMessages = new Quad();
			$popoutFrame->add($itemQuadMessages);
			$itemQuadMessages->setPosition($itemSize * $itemPlace, 0);
			$itemQuadMessages->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMessages->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_OFF));
			$itemQuadMessages->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_ON));
			$itemQuadMessages->setAction(self::ML_ICON_MESSAGE_ID);
			$itemQuadMessages->addTooltipLabelFeature($descriptionLabel, "Enable messages");
		} else {
			// Message are ON
			$itemQuadMessages = new Quad();
			$popoutFrame->add($itemQuadMessages);
			$itemQuadMessages->setPosition($itemSize * $itemPlace, 0);
			$itemQuadMessages->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMessages->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_ON));
			$itemQuadMessages->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_OFF));
			$itemQuadMessages->setAction(self::ML_ICON_MESSAGE_ID);
			$itemQuadMessages->addTooltipLabelFeature($descriptionLabel, "Disable messages");
		}
		$itemPlace--;
		
		// RANKING ICON
		if ($this->hasDisabledRanking($playerLogin)) {
			// Ranking is OFF
			$itemQuadRanking = new Quad();
			$popoutFrame->add($itemQuadRanking);
			$itemQuadRanking->setPosition($itemSize * $itemPlace, 0);
			$itemQuadRanking->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadRanking->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_OFF));
			$itemQuadRanking->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_ON));
			$itemQuadRanking->setAction(self::ML_ICON_RANKING_ID);
			$itemQuadRanking->addTooltipLabelFeature($descriptionLabel, "Enable ranking");
		} else {
			// Ranking is ON
			$itemQuadRanking = new Quad();
			$popoutFrame->add($itemQuadRanking);
			$itemQuadRanking->setPosition($itemSize * $itemPlace, 0);
			$itemQuadRanking->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadRanking->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_ON));
			$itemQuadRanking->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_OFF));
			$itemQuadRanking->setAction(self::ML_ICON_RANKING_ID);
			$itemQuadRanking->addTooltipLabelFeature($descriptionLabel, "Disable ranking");
		}
		$itemPlace--;
		
		// ENABLE/DISABLE MUSIC ICON
		if ($this->hasDisabledMusic($playerLogin)) {
			// Music is OFF
			$itemQuadMusic = new Quad();
			$popoutFrame->add($itemQuadMusic);
			$itemQuadMusic->setPosition($itemSize * $itemPlace, 0);
			$itemQuadMusic->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMusic->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_OFF));
			$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_ON));
			$itemQuadMusic->setAction(self::ML_ICON_MUSIC_ID);
			$itemQuadMusic->addTooltipLabelFeature($descriptionLabel, "Enable music");
		} else {
			// Music is ON
			$itemQuadMusic = new Quad();
			$popoutFrame->add($itemQuadMusic);
			$itemQuadMusic->setPosition($itemSize * $itemPlace, 0);
			$itemQuadMusic->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMusic->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_ON));
			$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_OFF));
			$itemQuadMusic->setAction(self::ML_ICON_MUSIC_ID);
			$itemQuadMusic->addTooltipLabelFeature($descriptionLabel, "Disable music");
		}
		$itemPlace--;
		
		// ENABLE/DISABLE SOUND ON DEMAND ICON
		if ($this->hasDisabledSOD($playerLogin)) {
			// SOD is OFF
			$itemQuadMusic = new Quad();
			$popoutFrame->add($itemQuadMusic);
			$itemQuadMusic->setPosition($itemSize * $itemPlace, 0);
			$itemQuadMusic->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMusic->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOD_OFF));
			$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOD_ON));
			$itemQuadMusic->setAction(self::ML_ICON_SOD_ON_OFF_ID);
			$itemQuadMusic->addTooltipLabelFeature($descriptionLabel, "Enable Sound On Demand");
		} else {
			// SOD is ON
			$itemQuadMusic = new Quad();
			$popoutFrame->add($itemQuadMusic);
			$itemQuadMusic->setPosition($itemSize * $itemPlace, 0);
			$itemQuadMusic->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMusic->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOD_ON));
			$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOD_OFF));
			$itemQuadMusic->setAction(self::ML_ICON_SOD_ON_OFF_ID);
			$itemQuadMusic->addTooltipLabelFeature($descriptionLabel, "Disable Sound On Demand");
		}
		$itemPlace--;
		
		// PLAY/STOP MUSIC ICON
		// Display icon only for ADMIN or greater
		// TO DO : replace with $player->isAdmin() ?
		if ($this->maniaControl->getAuthenticationManager()->checkRight($player, AuthenticationManager::AUTH_LEVEL_ADMIN)) {
			// Button displayed only if a link is set
			if ($this->MUSIC_LINK != '') {
				if ($this->musicIsPlaying == true) {
					// "Stop" button
					$itemQuadPlay = new Quad();
					$popoutFrame->add($itemQuadPlay);
					$itemQuadPlay->setPosition($itemSize * $itemPlace, 0);
					$itemQuadPlay->setSize($itemSize * $scale, $itemSize * $scale);
					$itemQuadPlay->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_STOP));
					$itemQuadPlay->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_PLAY));
					$itemQuadPlay->setAction(self::ML_ICON_PLAY_ID);
					$itemQuadPlay->addTooltipLabelFeature($descriptionLabel, "Stop music");
				} else {
					// "Play" button
					$itemQuadPlay = new Quad();
					$popoutFrame->add($itemQuadPlay);
					$itemQuadPlay->setPosition($itemSize * $itemPlace, 0);
					$itemQuadPlay->setSize($itemSize * $scale, $itemSize * $scale);
					$itemQuadPlay->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_PLAY));
					$itemQuadPlay->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_STOP));
					$itemQuadPlay->setAction(self::ML_ICON_PLAY_ID);
					$itemQuadPlay->addTooltipLabelFeature($descriptionLabel, "Play music");
				}
				$itemPlace--;
			}
		}
		
		// "SOUND ON DEMAND" ICON
		// Display icon only for ADMIN or greater
		// TO DO : replace with $player->isAdmin() ?
		if ($this->maniaControl->getAuthenticationManager()->checkRight($player, AuthenticationManager::AUTH_LEVEL_ADMIN)) {
			// "SoD" button
			$itemQuadSoD = new Quad();
			$popoutFrame->add($itemQuadSoD);
			$itemQuadSoD->setPosition($itemSize * $itemPlace, 0);
			$itemQuadSoD->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadSoD->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOD));
			$itemQuadSoD->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOD_MOVER));
			$itemQuadSoD->setAction(self::ML_ICON_SOD_MENU_ID);
			$itemQuadSoD->addTooltipLabelFeature($descriptionLabel, "Open Sound On Demand");
		}
		$itemPlace--;
		
		// Send ManiaLink to the player
		$this->sendManiaLink($maniaLink, 'player', $playerLogin, 'icon');
	}
	
	/*
	 * Remove the Menu Icons
	 */
	public function removeMenuIcon() {
		$this->maniaControl->getManialinkManager()->hideManialink(self::ML_ICON_MENU_ID);
	}
	
	
	
	
	/**
	 * Function handling the pressing of the SOUND icon, to activate/desactive sounds
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_sounds(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledSoundsPlayers" list
		if ($this->hasDisabledSounds($playerLogin)) {
			// Sending message to player
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sounds enabled.', $playerLogin, false);
			$this->sendInformation('Sounds enabled.', $playerLogin);
			// Remove player from the list
			$key = array_search($playerLogin, $this->disabledSoundsPlayers);
			unset($this->disabledSoundsPlayers[$key]);
		}
		else {
			// Sending message to player
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sounds disabled.', $playerLogin, false);
			$this->sendInformation('Sounds enabled.', $playerLogin);
			// Add player to the list
			$this->disabledSoundsPlayers[] = $playerLogin;
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_SOUNDS_PLAYERS  , implode(";", $this->disabledSoundsPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player, true);
	}
	
	
	
	/**
	 * Function handling the pressing of the MESSAGE icon, to activate/desactive messages
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_messages(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledMessagesPlayers" list
		if ($this->hasDisabledMessages($playerLogin)) {
			// Sending message to player
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Messages enabled.', $playerLogin, false);
			$this->sendInformation('Messages enabled.', $playerLogin);
			// Remove player from the list
			$key = array_search($playerLogin, $this->disabledMessagesPlayers);
			unset($this->disabledMessagesPlayers[$key]);
		}
		else {
			// Sending message to player
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Messages disabled.', $playerLogin, false);
			$this->sendInformation('Messages enabled.', $playerLogin);
			// Add player to the list
			$this->disabledMessagesPlayers[] = $playerLogin;
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_MESSAGES_PLAYERS, implode(";", $this->disabledMessagesPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player, true);
	}
	
	
	
	/**
	 * Function handling the pressing of the RANKING icon, to activate/desactive ranking
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_ranking(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledRankingPlayers" list
		if ($this->hasDisabledRanking($playerLogin)) {
			// Sending message to player
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Ranking enabled.', $playerLogin, false);
			$this->sendInformation('Ranking enabled.', $playerLogin);
			// Remove player from the list
			$key = array_search($playerLogin, $this->disabledRankingPlayers);
			unset($this->disabledRankingPlayers[$key]);
			// Display the RANKING to the player
			$this->ranking();
		}
		else {
			// Sending message to player
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Ranking disabled.', $playerLogin, false);
			$this->sendInformation('Ranking disabled.', $playerLogin);
			// Add player to the list
			$this->disabledRankingPlayers[] = $playerLogin;
			// Remove the RANKING widget to the player.
			$this->maniaControl->manialinkManager->hideManialink(self::ML_RANKING_ID, $playerLogin);
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_RANKING_PLAYERS , implode(";", $this->disabledRankingPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player, true);
	}
	
	
	
	/**
	 * Function handling the pressing of the MUSIC icon, to activate/desactive music
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_music(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledMusicPlayers" list
		if ($this->hasDisabledMusic($playerLogin)) {
			// Can't play music if already playing
			if ($this->musicIsPlaying == true) {
				// Sending message to player
				//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music enabled but can\'t be played while it\'s already playing.', $playerLogin, false);
				$this->sendInformation('Music enabled but can\'t be played while it\'s already playing.', $playerLogin);
			} else {
				// Sending message to player
				//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music enabled.', $playerLogin, false);
				$this->sendInformation('Music enabled.', $playerLogin);
			}
			// Remove player from the list
			// NB : can't play the music which is already playing, it will start the next time
			$key = array_search($playerLogin, $this->disabledMusicPlayers);
			unset($this->disabledMusicPlayers[$key]);
		}
		else {
			// Stop the music if already playing
			if ($this->musicIsPlaying == true) {
				// Sending message to player
				//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music disabled and stopped.', $playerLogin, false);
				$this->sendInformation('Music disabled and stopped.', $playerLogin);
				// Stop playing the music
				$this->stopMusic($playerLogin);
			}
			// No music to stop
			else {
				// Sending message to player
				//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music disabled.', $playerLogin, false);
				$this->sendInformation('Music disabled.', $playerLogin);
			}
			// Add player to the list
			$this->disabledMusicPlayers[] = $playerLogin;
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_MUSIC_PLAYERS   , implode(";", $this->disabledMusicPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player, true);
	}
	
	
	
	/**
	 * Function handling the pressing of the PLAY icon, to play/stop music
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function play_stop_music(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// MUSIC IS PLAYING : STOP IT
		if ($this->musicIsPlaying == true) {
			// Sending message to everyone
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music stopped.', null, false);
			$this->sendInformation('Music stopped.', null);
			// Stop playing the music
			$this->stopMusic(null);
			// Music is now disabled
			$this->musicIsPlaying = false;
		}
		// MUSIC IS STOPPED : START IT ?
		else {
			// Start only if music link exist
			if ($this->MUSIC_LINK == '') {
				//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Please set a music link in SEM parameter "MUSIC link".', $playerLogin, false);
				$this->sendInformation('Please set a music link in SEM parameter "MUSIC link".', $playerLogin);
			}
			// Play music
			else {
				// Sending message to everyone
				//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Playing music.', null, false);
				$this->sendInformation('Playing music.', null);
				// Start playing the music
				$this->playMusic();
				// Music is now enable
				$this->musicIsPlaying = true;
			}
		}
		
		// Update Menu Icons
		$this->addMenuIconToAll(true);
	}
	
	
	
	/**
	 * Function handling the pressing of the SOUND_ON_DEMAND icon
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function open_sound_on_demand(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		$this->toLog('dans open_sound_on_demand()');
		
		// NB : using ManialinkManager::MAIN_MLID to allow ManialinkManager::ACTION_CLOSEWIDGET to work.
		$maniaLink = new ManiaLink(ManialinkManager::MAIN_MLID);
		
		$xPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_SOUND_ON_DEMAND_POS_X);
		$yPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_SOUND_ON_DEMAND_POS_Y);
		$scale	= 0.7;
		$quadStyle		= $this->maniaControl->manialinkManager->getStyleManager()->getDefaultMainWindowStyle();
		$quadSubstyle	= $this->maniaControl->manialinkManager->getStyleManager()->getDefaultMainWindowSubStyle();
		
		// Count how many sounds have to be displayed
		$SOD_number = 0;
		for ($i = 1; $i <= 99 ; $i++) {
			$sound_2digits = sprintf("%02u",$i);
			if (!isset($this->{"SOUND_" . $sound_2digits . "_NAME"})) {
				break;
			}
			$SOD_number++;
		}
		
		// If no SOD to display, add one line for the information message
		$SOD_to_display = true;
		if ($SOD_number == 0) {
			$SOD_number++;
			$SOD_to_display = false;
		}
		
		// Set some working variables
		$column_number	= intval(($SOD_number - 1) / 25 + 1);
		$column_width	= intval(50 * $scale); // 50 is approximately the length of 20 characters (the maximum allowed)
		$table_width	= intval($column_width * $column_number);
		if ($column_number >= 2) { $height = intval(138 * $scale ); }		// Definated height for 2 or more columns
		else { $height = intval(5 * $SOD_number * $scale + 13 * $scale); }	// Dynamic height if only 1 column
		$offset       = 2;
		$offset_sod   = 5;
		$column_offset_X = 0;
		$column_offset_Y = 0;
		
		// Calculate the center of the frame, as $xPos/$yxPos is the top left corner of the frame and setPosition(x,y) expects the center of the frame in x/y.
		$xPos_center = $xPos + $table_width / 2; 
		$yPos_center = $yPos - $height / 2;
		
		// DEBUG
		$this->toLog("SOUND ON DEMAND : SOD_number=$SOD_number / column_number=$column_number");
		$this->toLog("SOUND ON DEMAND : center=$xPos/$yPos - top_left=$xPos_center/$yPos_center - table_width=$table_width - column_width=$column_width - height=$height");
		
		// Main frame
		$frame = new Frame();
		$maniaLink->add($frame);
		//$frame->setPosition($xPos, $yPos, 0); // NB : 0,0 is the middle of the frame.
		$frame->setPosition($xPos_center, $yPos_center, 0); // NB : 0,0 is the middle of the frame.
		$frame->setSize($table_width, $height);
		
		// Background of Main frame
		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		$backgroundQuad->setSize($table_width, $height);
		$backgroundQuad->setStyles($quadStyle, $quadSubstyle);
			
		// "SOUND ON DEMAND" message
		$messageLabel_0 = new Label_Button();
		$frame->add($messageLabel_0);
		//$messageLabel_0->setPosition(-15, $height / 2 - $offset, 1); // NB : 0,0 is the middle of the frame. "-15" to center the message.
		$messageLabel_0->setPosition(-21 * $scale, $height / 2 - $offset, 1); // NB : 0,0 is the middle of the frame. "-15" to center the message.
		$messageLabel_0->setScale($scale * 1.2);
		$messageLabel_0->setAlign('left', 'top');
		$messageLabel_0->setText('$b$fffSOUND ON DEMAND');
		$messageLabel_0->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Close Quad (X)
		$closeQuad = new Quad_Icons64x64_1();
		$frame->add($closeQuad);
		$closeQuad->setPosition($table_width / 2 - $offset, $height / 2 - $offset, 2); // NB : 0,0 is the middle of the frame
		$closeQuad->setSize(6, 6);
		$closeQuad->setSubStyle(Quad_Icons64x64_1::SUBSTYLE_QuitRace);
		$closeQuad->setAction(ManialinkManager::ACTION_CLOSEWIDGET);
		
		// Display an information message if no SOD is set
		if (!$SOD_to_display) {
			$messageLabel_0 = new Label_Button();
			$frame->add($messageLabel_0);
			$messageLabel_0->setPosition(-1 * $table_width / 2 + $offset + $column_offset_X, $height / 2 - 5 * $i * $scale - $offset_sod + $column_offset_Y, $i + 2);
			$messageLabel_0->setScale($scale);
			$messageLabel_0->setAlign('left', 'center');
			$messageLabel_0->setText("No sound to play.");
			$messageLabel_0->setStyle(Label_Text::STYLE_TextStaticSmall);
		}
		else {
			// Display all Sounds On Demand
			for ($i = 1; $i <= 99 ; $i++) {
				$sound_2digits = sprintf("%02u",$i);
				
				// NB : global variables to use
				// $this->{"SOUND_" . $sound_2digits . "_NAME"};
				// $this->{"SOUND_" . $sound_2digits . "_DESCRIPTION"};
				// $this->{"SOUND_" . $sound_2digits . "_LINK"};
				
				// Check is global variable exists
				if (!isset($this->{"SOUND_" . $sound_2digits . "_NAME"})) {
					break;
				}
				
				// New column every 25 lines
				if ($i % 25 == 1 && $i != 1) {
					$column_offset_X += $column_width;
					$column_offset_Y += 5 * 25 * $scale;
				}
				
				// Display the Sound on Demand
				$messageLabel_2 = new Label_Button();
				$frame->add($messageLabel_2);
				$messageLabel_2->setPosition(-1 * $table_width / 2 + $offset + $column_offset_X, $height / 2 - 5 * $i * $scale - $offset_sod + $column_offset_Y, $i + 2);
				$messageLabel_2->setScale($scale);
				$messageLabel_2->setAlign('left', 'center');
				$messageLabel_2->setText($sound_2digits . ' : ' . $this->{"SOUND_" . $sound_2digits . "_NAME"});
				$messageLabel_2->setStyle(Label_Text::STYLE_TextStaticSmall);
				$messageLabel_2->setAction(self::ML_SOUND_ON_DEMAND_ID . "." . $sound_2digits);
				
				// DESCRIPTION LABEL
				$descriptionFrame = new Frame();
				$frame->add($descriptionFrame);
				$descriptionFrame->setPosition($table_width / 2 + $offset, $height / 2 - 5 * $i * $scale - $offset_sod + $column_offset_Y, $i + 102);
				$descriptionFrame->setHAlign($descriptionFrame::LEFT);
				
				// NB : I can't find how to set a background to the description label (and display the background only when the label is displayed)
				/*
				$descriptionQuad = new Quad(); /// background
				$descriptionFrame->add($descriptionQuad);
				$descriptionQuad->setSize($column_width / 10, $height / 10);
				$descriptionQuad->setStyles($quadStyle, $quadSubstyle);
				*/
				$descriptionLabel = new Label();
				$descriptionFrame->add($descriptionLabel);
				$descriptionLabel->setAlign($descriptionLabel::LEFT, $descriptionLabel::CENTER);
				$descriptionLabel->setSize(30, 2);
				$descriptionLabel->setTextSize($scale * 1.2);
				$descriptionLabel->setVisible(true);
				$descriptionLabel->setTextColor('fff');
				$messageLabel_2->addTooltipLabelFeature($descriptionLabel, "\$i" . $this->{"SOUND_" . $sound_2digits . "_DESCRIPTION"}); // TEST
			}
		}
		
		// Send ManiaLink to the player
		$this->sendManiaLink($maniaLink, 'player', $playerLogin, 'menu');
	}
	
	
	/**
	 * Function handling the pressing of the SOUND_ON_DEMAND icon
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_sound_on_demand(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledSoundOnDemandPlayers" list
		if ($this->hasDisabledSOD($playerLogin)) {
			// Sending message to player
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sound On Demand enabled.', $playerLogin, false);
			$this->sendInformation('Sound On Demand enabled.', $playerLogin);
			// Remove player from the list
			$key = array_search($playerLogin, $this->disabledSoundOnDemandPlayers);
			unset($this->disabledSoundOnDemandPlayers[$key]);
		}
		else {
			// Sending message to player
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sound On Demand disabled.', $playerLogin, false);
			$this->sendInformation('Sound On Demand disabled.', $playerLogin);
			// Add player to the list
			$this->disabledSoundOnDemandPlayers[] = $playerLogin;
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_SOUND_ON_DEMAND_PLAYERS, implode(";", $this->disabledSoundOnDemandPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player, true);
	}
	
	
	
	/**
	 *  Play a Sound On Demand depending on a callback name
	 *
	 * @param array  $chatCallback
	 */
	public function playSoundOnDemand(array $callback) {
		// This function is reserved to self::ML_SOUND_ON_DEMAND_ID callbacks
		$callbackId = $callback[1][2];
		$boolSetting = (strpos($callbackId, self::ML_SOUND_ON_DEMAND_ID) === 0);
		if (!$boolSetting) { return; }
		
		// Get the SoD id from the callback name (cf open_sound_on_demand() for creation of callback)
		$actionArray = explode(".", $callbackId);
		$soundId = intval($actionArray[4]);
		
		// Get some other parameters
		$player     	= $this->maniaControl->getPlayerManager()->getPlayer($callback[1][1]);
		$playerLogin	= $player->login;
		$playerNickname = $player->nickname;
		$SOD_global_delay	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_SOUND_ON_DEMAND_GLOBAL_DELAY);
		$SOD_user_delay		= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_SOUND_ON_DEMAND_USER_DELAY);
		
		// DEBUG
		$this->toLog("Global delay  = $SOD_global_delay / User delay = $SOD_user_delay");
		
		// Global delay elapsed ?
		if ($this->SOD_global_delay_elapsed == false) {
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Can\'t play Sound On Demand at the moment. Global delay  (' . $SOD_global_delay . ' ms) not elapsed. Try again later.', $playerLogin, false);
			//$this->sendInformation('Can\'t play Sound On Demand at the moment. Global delay  (' . $SOD_global_delay . ' ms) not elapsed. Try again later.', $playerLogin);
			$this->sendInformation("Please wait $SOD_global_delay ms between each SOD played by an admin.", $playerLogin);
			return;
		}
		// User delay elapsed ?
		if (in_array($playerLogin, $this->SOD_spamList)) {
			//$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Can\'t play Sound On Demand at the moment. User delay (' . $SOD_user_delay . ' ms) not elapsed. Try again later.', $playerLogin, false);
			//$this->sendInformation('Can\'t play Sound On Demand at the moment. User delay (' . $SOD_user_delay . ' ms) not elapsed. Try again later.', $playerLogin);
			$this->sendInformation("Please wait $SOD_user_delay ms between each SOD you play.", $playerLogin);
			return;
		}
		
		// Get informations on SOD
		$sound_2digits = sprintf("%02u",$soundId);
		$name				= $this->{"SOUND_" . $sound_2digits . "_NAME"};
		$link				= $this->{"SOUND_" . $sound_2digits . "_LINK"};
		$blockothersounds	= $this->{"SOUND_" . $sound_2digits . "_BLOCKOTHERSOUNDS"};
		$duration			= $this->{"SOUND_" . $sound_2digits . "_DURATION"};
		
		// Trying to play SOD
		$rc = $this->playSound('all', $link, $blockothersounds, $duration, $playerLogin, 'sod');
		
		// SOD is played (not blocked by an other sound)
		if ($rc) {
			//$this->sendInformation('Playing Sound On Demand \'' . $name . '\' by ' . $playerNickname, null);
			$SOD_message_activated = $this->maniaControl->settingManager->getSettingValue($this, self::SEM_SOUND_ON_DEMAND_MESSAGE_ACTIVATED);
			if ($SOD_message_activated) { $this->sendInformation("Playing Sound On Demand '$name' by $playerNickname", null); }
			
			// DEBUG
			$this->toLog("link=$link / blockothersounds=$blockothersounds / duration=$duration");
			
			// Add global delay
			$this->SOD_global_delay_elapsed = false;
			$this->maniaControl->timerManager->registerOneTimeListening($this, 'resetGlobalDelay', $SOD_global_delay);
			// Add user delay
			$this->SOD_spamList[] = $playerLogin;
			$this->maniaControl->timerManager->registerOneTimeListening($this, 'removePlayerFromSoundOnDemandSpamList', $SOD_user_delay);
		}
		// SOD blocked
		else {
			$this->sendInformation("Please wait the end of this SOD.", $playerLogin);
		}
	}
	
	/*
	 * Remove the first element of the SpamList.
	 *
	 */
	public function resetGlobalDelay() {
		$this->SOD_global_delay_elapsed = true;
		
		// DEBUG
		$this->toLog('Reseting global delay');
	}
	/*
	 * Remove the first element of the SpamList.
	 *
	 */
	public function removePlayerFromSoundOnDemandSpamList() {
		$user = array_shift($this->SOD_spamList);
		
		// DEBUG
		$this->toLog("Removing $user from SOD_spamList");
	}
	
	
	
	/*
	 * Get parameters value of $allowedTo and $link and try to play the sound.
	 *
	 */
	public function soundToPlay($self_enabled_to, $link, $blockothersounds, $duration, $authorLogin) {
		// Get settings from parameters
		$allowedTo	= $this->maniaControl->settingManager->getSettingValue($this, $self_enabled_to);
		
		// Play the sound if able
		$rc = $this->playSound($allowedTo, $link, $blockothersounds, $duration, $authorLogin, 'sound');
		return $rc;
	}
	
	
	
	/*
	 * Plays the sound "$link" if able
	 *
	 * Returns true if a sound has been played, false otherwise
	 *
	 */
	//public function playSound($allowedTo, $link, $authorLogin, $type) {
	public function playSound($allowedTo, $link, $blockothersounds, $duration, $authorLogin, $type) {
	
		// Volume
		if ($type == 'sod') {
			$volume	= $this->maniaControl->settingManager->getSettingValue($this, self::VOLUME_SOUND_ON_DEMAND);
		} else {
			$volume	= $this->maniaControl->settingManager->getSettingValue($this, self::VOLUME_SOUND);
		}
		
		// Don't play sound if any :
		//    - not allowed by $allowedTo
		//    - $link doesn't exist
		//    - sound author is IGNORED
		//    - a sound with "blockothersounds == true" is already playing and not finished yet
		// NB : Keep "===" !
		if ($allowedTo === false || $allowedTo == 'none' || empty($link) || $link == '' || $this->isIgnored($authorLogin) || $this->soundsBlocked == true) { return false; }
		
		// Create an audio element to play a sound with the FML
		// AUB v1.11
		/*
		if ($type == 'sod') {
			$maniaLink = new \FML\ManiaLink(self::ML_SOUND_ON_DEMAND_ID);
		} else {
			$maniaLink = new \FML\ManiaLink(self::ML_SOUND_ID);
		}
		*/
		$maniaLink = new \FML\ManiaLink(self::ML_SOUND_ID);
		$audio = new \FML\Controls\Audio();
		$maniaLink->add($audio);
		$audio->setVolume($volume);		//						  from FML/Types/Playable.php
		$audio->setPlay(1);				// Play automatically	- from FML/Types/Playable.php
		$audio->setLooping(0);			// Play only once		- from FML/Types/Playable.php
		$audio->setVisible(0);			// Popup not visible	- from FML/Controls/Control.php
		$audio->setData($link);
		
		// Send ManiaLink
		$this->sendManiaLink($maniaLink, $allowedTo, $authorLogin, $type);
		
		// Prevent other sounds to be played if "blockothersounds" is set for this sound
		if ($blockothersounds == 'true') {
			$this->soundsBlocked = true;
			$this->maniaControl->timerManager->registerOneTimeListening($this, 'resetSoundsBlocked', $duration);
		}
		
		return true;
	}
	
	/*
	 * Sounds are no more blocked
	 */
	public function resetSoundsBlocked() {
		$this->soundsBlocked = false;
		
		// DEBUG
		$this->toLog('resetSoundsBlocked()');
	}
	
	
	
	/**
	 * Display a message during a delay
	 *
	 */
	public function showMessage($message, $self_enabled_to, $authorLogin) {
		// Remove the previous timer listener to keep the delay before removing the message
        $this->maniaControl->timerManager->unregisterTimerListening($this, 'removeMessage');
		
		// Don't display message if author of sound is IGNORED
		if ($this->isIgnored($authorLogin)) { return false; }
		
		// Don't display message if not enabled
		$enabled = $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_ENABLED);
		//$this->toLog("showMessage enabled : " . $enabled);
		if ($enabled == false) { return false; }
		
		$maniaLink = new ManiaLink(self::ML_MESSAGE_ID);
		$allowedTo = $this->maniaControl->settingManager->getSettingValue($this, $self_enabled_to);
		
		// Main frame
		$frame = new Frame();
		$maniaLink->add($frame);
		
		$xPos		= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_POS_X);
		$yPos		= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_POS_Y);
		$scale		= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_SCALE);
		$duration	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_DURATION);
		$frame->setPosition($xPos, $yPos, 0);

		// Background
		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		
		// Message
		$messageLabel = new Label_Button();
		$frame->add($messageLabel);
		$messageLabel->setScale($scale);
		$messageLabel->setText($message);
		$messageLabel->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Send ManiaLink
		$this->sendManiaLink($maniaLink, $allowedTo, $authorLogin, 'message');
		
		// Remove the message after a delay
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'removeMessage', $duration);
	}
	
	/*
	 * Remove the message
	 */
	public function removeMessage() {
		$this->maniaControl->manialinkManager->hideManialink(self::ML_MESSAGE_ID);
	}
	
	
	
	/**
	 * Popup a widget with the killstreak ranking of 3 best players
	 */
	public function showRanking($killStreak_1_login, $killStreak_1_score, $killStreak_2_login, $killStreak_2_score, $killStreak_3_login, $killStreak_3_score) {
		// Ranking enabled ?
		$enabled = $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_ENABLED);
		
		// Nothing to display or RANKING not enabled ?
		if ($killStreak_1_login == null || $enabled == false) {
			// Remove the widget
			$this->maniaControl->manialinkManager->hideManialink(self::ML_RANKING_ID);
			return;
		}
		
		$maniaLink = new ManiaLink(self::ML_RANKING_ID);

		// Main frame
		$frame = new Frame();
		$maniaLink->add($frame);
		$xPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_POS_X);
		$yPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_POS_Y);
		$scale	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_SCALE);
		$frame->setPosition($xPos, $yPos, 0);

		// Background
		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		
		// Ranking message
		$messageLabel_0 = new Label_Button();
		$frame->add($messageLabel_0);
		$messageLabel_0->setPosition(1, 0, 1);
		$messageLabel_0->setScale($scale);
		$messageLabel_0->setAlign('right', 'center');
		$messageLabel_0->setText('$b$fff RANKING');
		$messageLabel_0->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Ranking 1st player
		$killStreak_1_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_1_login)->nickname;
		$messageLabel_1 = new Label_Button();
		$frame->add($messageLabel_1);
		$messageLabel_1->setPosition(1, -5, 2);
		$messageLabel_1->setScale($scale);
		$messageLabel_1->setAlign('right', 'center');
		$messageLabel_1->setText($killStreak_1_nickname . '$z$s$fff : ' . $killStreak_1_score . ' frags');
		$messageLabel_1->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Ranking 2nd player
		if ($killStreak_2_login != null) {
			$killStreak_2_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_2_login)->nickname;
			$messageLabel_2 = new Label_Button();
			$frame->add($messageLabel_2);
			$messageLabel_2->setPosition(1, -10, 3);
			$messageLabel_2->setScale($scale);
			$messageLabel_2->setAlign('right', 'center');
			$messageLabel_2->setText($killStreak_2_nickname . '$z$s$fff : ' . $killStreak_2_score . ' frags');
			$messageLabel_2->setStyle(Label_Text::STYLE_TextStaticSmall);
		}
		
		// Ranking 3rd player
		if ($killStreak_3_login != null) {
			$killStreak_3_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_3_login)->nickname;
			$messageLabel_3 = new Label_Button();
			$frame->add($messageLabel_3);
			$messageLabel_3->setPosition(1, -15, 4);
			$messageLabel_3->setScale($scale);
			$messageLabel_3->setAlign('right', 'center');
			$messageLabel_3->setText($killStreak_3_nickname . '$z$s$fff : ' . $killStreak_3_score . ' frags');
			$messageLabel_3->setStyle(Label_Text::STYLE_TextStaticSmall);
		}
		
		// Send ManiaLink to everyone (true) by default
		$this->sendManiaLink($maniaLink, true, null, 'ranking');
	}
	
	
	
	/*
	 * Check if music should be played. If so, plays the music.
	 */
	public function musicToPlay() {
		$MUSIC_ENABLED = $this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_ENABLED);
		if ($MUSIC_ENABLED == true) {
			// Play the music
			$this->playMusic();
			// Music is playing
			$this->musicIsPlaying = true;
			// Update menu for each player
			$this->addMenuIconToAll(false);	
		}
	}
	
	/*
	 * Plays music in loop
	 *
	 */
	public function playMusic() {
		$MUSIC_VOLUME	= $this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_VOLUME);
		
		// Don't play sounds if $MUSIC_LINK doesn't exist
		if ($this->MUSIC_LINK == '') { return; }
		
		// Tableau des fichiers à lire
		// TO DO : passer les variables $ftp* en paramètre du plugin
		// TO DO : ATTENTION !!!! Transforme un lien FTP en un lien HTTP : nécessite d'avoir les fichiers hébergés sur le serveur Web et avoir le bon répertoire à la connexion FTP (/var/www) afin que le lien soit cohérent
		/*
		Stocker les sons dans le répertoire Plugins/Lumpik/Sounds/ et le plugin va les chercher dedans
		
		$ftpServer = 'mm2monde.fr';
		$ftpLogin = 'user';
		$ftpPass = 'pass';
		$directory = 'Plugins/SoundEventsManager/Musiques/';
		$filesList = $this->ScanDirectory($ftpServer, $ftpLogin, $ftpPass, $directory);
		// Get a random file from the list
		if ($filesList != null) {
			$randomKey = array_rand($filesList);
			$this->MUSIC_LINK = "http://$ftpServer/$directory" . $filesList[$randomKey];
			$this->toLog("Random music link : $this->MUSIC_LINK");
		}
		*/
		/* FICHIERS EN LOCAL
		// NB : peut jouer un manialink avec un chemin local ? fonctionne avec les locators ?
		$filesList = $this->ScanLocalDirectory($directory);
		// Get a random file from the list
		if ($filesList != null) {
			$randomKey = array_rand($filesList);
			// TO DO $this->MUSIC_LINK = "http://$ftpServer/$directory" . $filesList[$randomKey];
			$this->toLog("Random music link : $this->MUSIC_LINK");
		}
		*/
		
		// Create an audio element to play a sound with the FML
		$maniaLink = new \FML\ManiaLink(self::ML_MUSIC_ID);
		$audio = new \FML\Controls\Audio();
		$maniaLink->add($audio);
		$audio->setVolume($MUSIC_VOLUME);	//						  from FML/Types/Playable.php
		$audio->setPlay(1);					// Play automatically	- from FML/Types/Playable.php
		$audio->setLooping(1);				// Play in loop			- from FML/Types/Playable.php
		$audio->setVisible(0);				// Popup not visible	- from FML/Controls/Control.php
		//$audio->setMusic($this->MUSIC_LINK);	// KO
		$audio->setData($this->MUSIC_LINK);
		
		// Send ManiaLink to everyone (true) by default
		$this->sendManiaLink($maniaLink, true, null, 'music');
	}
	
	/*
	 * Stop the music
	 */
	public function stopMusic($login) {
		$this->maniaControl->manialinkManager->hideManialink(self::ML_MUSIC_ID, $login);
	}
	
	
	
	/**
	 * Send a ManiaLink to [$login / team of $login / all], depending on $allowedTo, and if $type is allowed for receivers
	 * 
	 */
	public function sendManiaLink($maniaLink, $allowedTo, $login, $type) {
		// DEBUG
		//$this->toLog("sendManiaLink() allowedTo=$allowedTo");
		
		// Update allowedTo because "case true:" don't works...
		// NB : "==" : transforme toujours en "all". "===" ne fonctionne pas pour les sons enabled/disabled comme le BEGING_MAP / BEGIN_PLAYING
		if ($allowedTo === true || $allowedTo == 1) { $allowedTo = 'all'; }
		
		// DEBUG
		$this->toLog("sendManiaLink() allowedTo=$allowedTo - login=$login - type=$type");
		
		// RECEIVERS ?
		switch ($allowedTo) {
			// PLAYER
			case 'player':
				// Don't send if $login is not set
				if (!is_null($login)) {
					// Send ManiaLink only to non-ignored players and if type of ManiaLink is not disabled for this player
					if (!$this->isIgnored($login) && !$this->hasDisabledFromType($login, $type)) {
						// DEBUG
						//$this->toLog("sendManiaLink() sent : player ($login)");
						$this->maniaControl->manialinkManager->sendManialink($maniaLink, $login);
					}
				}
				break;
			// TEAM
			case 'team':
				// Team ID of $login
				$loginTeamId = $this->maniaControl->getPlayerManager()->getPlayer($login)->teamId;
				// DEBUG
				//$this->toLog("sendManiaLink() team of login $login : $loginTeamId");
				
				// Send ManiaLink only to teammates, non-ignored players and if type of ManiaLink is not disabled for this player
				foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
					$playerLogin = $player->login;
					$playerTeamId = $player->teamId;
					if ($loginTeamId == $playerTeamId && !$this->isIgnored($playerLogin) && !$this->hasDisabledFromType($playerLogin, $type)) {
						// DEBUG
						//$this->toLog("sendManiaLink() sent : team ($playerLogin)($playerTeamId)");
						$this->maniaControl->manialinkManager->sendManialink($maniaLink, $playerLogin);
					}
				}
				break;
			// ALL
			case 'all':
				// Send ManiaLink only to non-ignored players and if type of ManiaLink is not disabled for this player
				foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
					$playerLogin = $player->login;
					if (!$this->isIgnored($playerLogin) && !$this->hasDisabledFromType($playerLogin, $type)) {
						// DEBUG
						//$this->toLog("sendManiaLink() sent : all ($playerLogin)");
						$this->maniaControl->manialinkManager->sendManialink($maniaLink, $playerLogin);
					}
				}
				break;
		}
	}
	
	
	
	/**
	 * Send an information message to a player or everybody
	 * 
	 */
	public function sendInformation($message, $player) {
		$this->maniaControl->getChat()->sendInformation('[Sound Events Manager][' . $this->getVersion() . '] ' . $message, $player, false);
	}
	
	
	
	/*
	 * Return true is $login is ignored, false otherwise.
	 */
	public function isIgnored($login) {
		// Stop if there's no $login
		if ($login == null) { return false; }
		
		// Get ignore list
		$IGNORED_PLAYERS_LIST	= $this->maniaControl->settingManager->getSettingValue($this, self::IGNORED_PLAYERS_LIST);
		
		// Get the list of ignored players in an array
		$listeOfIgnoredPlayers = array();
		if (!empty($IGNORED_PLAYERS_LIST)) { $listeOfIgnoredPlayers = explode(";", $IGNORED_PLAYERS_LIST); }
		
		// Return if $login is ignored
		if (!empty($listeOfIgnoredPlayers) && in_array($login, $listeOfIgnoredPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled $type, false otherwise.
	 */
	public function hasDisabledFromType($login, $type) {
		$hasDisabledType = false;
		switch ($type) {
			case 'sound':
				if ($this->hasDisabledSounds($login)) { $hasDisabledType = true; }
				break;
			case 'message':
				if ($this->hasDisabledMessages($login)) { $hasDisabledType = true; }
				break;
			case 'ranking':
				if ($this->hasDisabledRanking($login)) { $hasDisabledType = true; }
				break;
			case 'music':
				if ($this->hasDisabledMusic($login)) { $hasDisabledType = true; }
				break;
			case 'sod':
				if ($this->hasDisabledSOD($login)) { $hasDisabledType = true; }
				break;
			case 'icon':
				// Always allow icons. Can't activate/desactive SEM without it.
				$hasDisabledType = false;
				break;
		}
		return $hasDisabledType;
	}
	
	/*
	 * Return true is $login has disabled plugin, false otherwise.
	 */
	public function hasDisabledSounds($login) {
		if (in_array($login, $this->disabledSoundsPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled messages, false otherwise.
	 */
	public function hasDisabledMessages($login) {
		if (in_array($login, $this->disabledMessagesPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled ranking, false otherwise.
	 */
	public function hasDisabledRanking($login) {
		if (in_array($login, $this->disabledRankingPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled music, false otherwise.
	 */
	public function hasDisabledMusic($login) {
		if (in_array($login, $this->disabledMusicPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled music, false otherwise.
	 */
	public function hasDisabledSOD($login) {
		if (in_array($login, $this->disabledSoundOnDemandPlayers)) { return true; }
		else { return false; }
	}
	
	
	
	/**
	 * Function handling on the connection of a player.
	 *
	 * @param Player $player
	 */
	public function onPlayerConnect(Player $player) {
		$login = $player->login;
		// Initialization of the player connected
		$this->playersScore[$login] = '0';
		// Play sound ?
		$this->soundToPlay(self::PLAYER_CONNECTED_ENABLED, $this->PLAYER_CONNECTED_LINK, $this->PLAYER_CONNECTED_BLOCKOTHERSOUNDS, $this->PLAYER_CONNECTED_DURATION, $login);
		// Display menu icons for this player
		$this->addMenuIcon($player, false);
		// Preload all sounds
		$this->preloadAllSounds($login);
	}
	
	/**
	 * Function handling on the disconnection of a player.
	 *
	 * @param array $callback
	 */
	public function onPlayerDisconnect(Player $player) {
		$login = $player->login;
		// Play sound ?
		$this->soundToPlay(self::PLAYER_DISCONNECTED_ENABLED, $this->PLAYER_DISCONNECTED_LINK, $this->PLAYER_DISCONNECTED_BLOCKOTHERSOUNDS, $this->PLAYER_DISCONNECTED_DURATION, $login);
		// Removing informations from this player
		unset($this->playersScore[$login]);
		// Update RANKING
		$this->ranking();
	}
	
	
	
	/**
	 * Function called when starting a new map
	 */
	public function onBeginMap() {
		// Initialisation players's score
		$this->initializePlayersScore();
		
		// Music to play ?
		$musicDelay = $this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_DELAY);
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'musicToPlay', $musicDelay);
		
		// Update ranking
		$this->ranking();
		
		// Sound ?
		$beginMapDelay = $this->maniaControl->settingManager->getSettingValue($this, self::BEGIN_MAP_DELAY);
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'playBeginMapSound', $beginMapDelay);
	}
	
	/*
	 * Play BEGIN_MAP sound
	 */
	public function playBeginMapSound() {
		// Sound to play ?
		$this->soundToPlay(self::BEGIN_MAP_ENABLED, $this->BEGIN_MAP_LINK, $this->BEGIN_MAP_BLOCKOTHERSOUNDS, $this->BEGIN_MAP_DURATION, null);
	}
	
	
	
	/**
	 * Function called when starting a new round
	 */
	public function onBeginPlaying() {
		$beginPlayingDelay = $this->maniaControl->settingManager->getSettingValue($this, self::BEGIN_PLAYING_DELAY);
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'playBeginPlayingSound', $beginPlayingDelay);
		
		// Reset killstreak ?
		$RESET_ON_NEW_ROUND = $this->maniaControl->settingManager->getSettingValue($this, self::RESET_ON_NEW_ROUND);
		if ($RESET_ON_NEW_ROUND) {
			// Reset scores
			$this->initializePlayersScore();
			
			// Update ranking
			$this->ranking();
		}
	}
	
	/*
	 * Play BEGING_PLAYING sound
	 */
	public function playBeginPlayingSound() {
		$this->soundToPlay(self::BEGIN_PLAYING_ENABLED, $this->BEGIN_PLAYING_LINK, $this->BEGIN_PLAYING_BLOCKOTHERSOUNDS, $this->BEGIN_PLAYING_DURATION, null);
	}
	
	
	
	/**
	 * Function called when some captures a pole
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/CaptureStructure.php
	public function onCapture(CaptureStructure $structure) {
		foreach ($structure->getPlayerArray() as $player) {
			$this->toLog($player->nickname . '$z$s$fff (' . $player->teamId . ') : Nice capture !');
			
			// BLUE/RED TEAM SCORES
			$this->playerTeamId = $player->teamId;
			if (!$this->isIgnored($player->login)) {
				$teamScoresDelay = $this->maniaControl->settingManager->getSettingValue($this, self::TEAM_SCORES_DELAY);
				$this->maniaControl->timerManager->registerOneTimeListening($this, 'playTeamScoresSound', $teamScoresDelay);
			}
		}
	}
	
	/*
	 * Play BLUE/RED TEAM SCORES sound
	 */
	public function playTeamScoresSound() {
		if ($this->playerTeamId == 0) {
			$this->soundToPlay(self::TEAM_SCORES_ENABLED_TO, $this->BLUE_TEAM_SCORES_LINK, $this->BLUE_TEAM_SCORES_BLOCKOTHERSOUNDS, $this->BLUE_TEAM_SCORES_DURATION, null);
		}
		else {
			$this->soundToPlay(self::TEAM_SCORES_ENABLED_TO, $this->RED_TEAM_SCORES_LINK, $this->RED_TEAM_SCORES_BLOCKOTHERSOUNDS, $this->RED_TEAM_SCORES_DURATION, null);
		}
	}
	
	
	
	/**
	 * Function called when someone BACKSPACE (+ suppr ?)
	 */
	public function onRequestRespawn(Player $player) {
		$playerNickname	= $player->nickname;
		$playerLogin	= $player->login;
		// Reset player's score
		$this->playersScore[$playerLogin] = '0';
		// Sound to play ?
		$rc = $this->soundToPlay(self::BACKSPACE_ENABLED_TO, $this->BACKSPACE_LINK, $this->BACKSPACE_BLOCKOTHERSOUNDS, $this->BACKSPACE_DURATION, $playerLogin);
		// Message to display ?
		if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Backspace !', self::BACKSPACE_ENABLED_TO, $playerLogin); }
		// Update RANKING
		$this->ranking();
		
		// Score of teams - TO DO
		// Récupérer les paramètres du mode (le temps de chaque phase) et le score de chaque équipe
		// 		S_TimeLimitNormal
		// 		S_TimeLimitReduction
		// 		S_TimeLimitInstant
		//	
		//$this->maniaControl->settingManager->getSetting
	}
	
	
	
	/*
	 * Function called when a player hit another player
	 */
	/*
	public function onShoot(Player $player) {
		$message = $player->nickname . '$z$s$fff : shoot';
		//$this->toLog($message);
		// Message to display ?
		$this->showMessage($message, XXX_ENABLED_TO, $player->login);
	}
	*/
	
	
	
	/*
	// Calculates skalar of two vectors
	Real Skalar(Vec3 _Vec1, Vec3 _Vec2) {
		return _Vec1[0]*_Vec2[0] + _Vec1[1]*_Vec2[1] + _Vec1[2]*_Vec2[2];
	}

	// Plays headshot sound on headshot
	Void DoHeadShot(CSmModeEvent _Event) {
		if (!C_AnnounceHeadshots || _Event == Null || _Event.Shooter == Null || _Event.Victim == Null) return;
		if (_Event.WeaponNum != GetWeaponNum(CSmMode::EWeapon::Laser)) return;
		
		declare UI <=> UIManager.GetUI(_Event.Shooter);
		if (UI == Null) return;
		
		declare HeadPosi = <_Event.Victim.Position[0] + _Event.Victim.AimDirection[0] * 0.2,
							_Event.Victim.Position[1] + 1.5,
							_Event.Victim.Position[2] + _Event.Victim.AimDirection[2] * 0.2>;
		declare ShootPosi = _Event.Shooter.Position + <0., 1.35, 0.>;
		declare ShootDir = _Event.Shooter.AimDirection;
		
		declare a1 = Skalar(ShootDir, ShootPosi);
		declare a2 = Skalar(ShootDir, ShootDir);
		declare a3 = Skalar(ShootDir, -HeadPosi);
		
		declare gamma = (a1+a3)/(-1*a2);
		declare stutz = ShootPosi + (gamma*ShootDir);
		
		declare distance = MathLib::Distance(HeadPosi, stutz);
		if (distance > 0.2) return;
		
		Message::SendStatusMessage(_Event.Shooter, _("Headshot!"), 2000, 0);
		UI.ManialinkPage = """//{{{Now}}}
			<audio looping="0" hidden="1" play="1" data="http://dl.dropboxusercontent.com/u/58886549/ManiaPlanet/Media/headshot0.ogg"/>""";
	}
	*/
	
	public function array_add_array($array1, $array2) {
		return array($array1[0] + $array2[0], $array1[1] + $array2[1], $array1[2] + $array2[2]);
	}
	
	public function array_add_int($array1, $int) {
		return array($array1[0] + $int, $array1[1] + $int, $array1[2] + $int);
	}
	
	public function array_mult_array($array1, $array2) {
		return array($array1[0] * $array2[0], $array1[1] * $array2[1], $array1[2] * $array2[2]);
	}
	
	public function array_mult_int($array1, $int) {
		return array($array1[0] * $int, $array1[1] * $int, $array1[2] * $int);
	}
	
	public function Skalar($array1, $array2) {
		return $array1[0]*$array2[0] + $array1[1]*$array2[1] + $array1[2]*$array2[2];
	}
	
	// KO : can't get victim->Position and victim->AimDirection from ManiaControl callbacks
	/*
	public function DoHeadShot($weapon, $shooter, $victim) {
		
		// Only laser can headshot, check other parameters
		if ($weapon != 1 || $shooter == null || $victim == null) { return; }
		
		$pos0 = $victim->Position[0];
		$pos1 = $victim->Position[1];
		$pos2 = $victim->Position[2];
		$this->toLog("victim->Position[]=  $pos0 - $pos1 - $pos2");
		
		/*
		// Messy computing...
		$HeadPosi	= array($victim->Position[0] + $victim->AimDirection[0] * 0.2,
							$victim->Position[1] + 1.5,
							$victim->Position[2] + $victim->AimDirection[2] * 0.2);
		var_dump($HeadPosi); // DEBUG
		
		$HeadPosi_Negatif = $this->array_mult_int($HeadPosi, -1);
		var_dump($HeadPosi_Negatif); // DEBUG
		
		$ShootPosi	= array($shooter->Position[0] + 0,
							$shooter->Position[1] + 1.35,
							$shooter->Position[2] + 0);
		var_dump($ShootPosi); // DEBUG
		
		$ShootDir	= $shooter->AimDirection;
		var_dump($ShootDir); // DEBUG
		
		$a1 = Skalar($ShootDir, $ShootPosi);
		$a2 = Skalar($ShootDir, $ShootDir);
		$a3 = Skalar($ShootDir, $HeadPosi_Negatif);
		var_dump($a1); // DEBUG
		var_dump($a2); // DEBUG
		var_dump($a3); // DEBUG
		
		$gamma = ($a1+$a3)/(-1*$a2);
		var_dump($gamma); // DEBUG
		
		$stutz = $this->array_add_array($ShootPosi, $this->array_mult_int($ShootDir, $gamma));
		var_dump($stutz); // DEBUG
		
		$carre1 = pow($HeadPosi[0] - $stutz[0], 2);
		$carre2 = pow($HeadPosi[1] - $stutz[1], 2);
		$carre3 = pow($HeadPosi[2] - $stutz[2], 2);
		var_dump($carre1); // DEBUG
		var_dump($carre2); // DEBUG
		var_dump($carre3); // DEBUG
		
		$distance = sqrt($carre1 + $carre2 + $carre3);
		var_dump($distance); // DEBUG
		
		if ($distance > 0.2) { return; }
		
		// Affichage "headshot"
		$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Headshot !', null, false);
		$this->sendInformation("Headshot !", null);
		
		
		//Message::SendStatusMessage(_Event.Shooter, _("Headshot!"), 2000, 0);
		//UI.ManialinkPage = """//{{{Now}}}
		//	<audio looping="0" hidden="1" play="1" data="http://dl.dropboxusercontent.com/u/58886549/ManiaPlanet/Media/headshot0.ogg"/>""";
	}
	*/
	
	
	
	/*
	 * Function called when a player hit another player
	 */
	public function onHit(PlayerHitStructure $structure) {
		$shooter = $structure->getShooter();
		$victim = $structure->getVictim();
		$damage = $structure->getDamage();
		$shooterPoints = $structure->getShooterPoints();
		$weapon = $structure->getWeapon();
		$hitDistance = $structure->getHitDistance();
		
		//$this->toLog('onHit() : ' . $shooter->nickname . '$z$s$fff hit ' . $victim->nickname . '$z$s$fff with weapon ' . $weapon);
		
		// Play sound (?) depending on the weapon used
		// DOC : https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/Weapons.php
		switch ($weapon) {
			// LASER
			case '1':
				// Play sound ?
				$this->soundToPlay(self::HIT_LASER_ENABLED_TO, $this->HIT_LASER_LINK, $this->HIT_LASER_BLOCKOTHERSOUNDS, $this->HIT_LASER_DURATION, $shooter->login);
				//$this->maniaControl->timerManager->registerOneTimeListening($this, 'playHitByLaserSound', 500);
				// NB : To use a timer, i need to create global variable to store $shooter->login ==> what if another hit occurs before the 1st one resolve ? Use a array of login ?
				break;
			// ROCKET
			case '2':
				// Play sound ?
				$this->soundToPlay(self::HIT_ROCKET_ENABLED_TO, $this->HIT_ROCKET_LINK, $this->HIT_ROCKET_BLOCKOTHERSOUNDS, $this->HIT_ROCKET_DURATION, $shooter->login);
				break;
			// NUCLEUS
			case '3':
				// Play sound ?
				$this->soundToPlay(self::HIT_NUCLEUS_ENABLED_TO, $this->HIT_NUCLEUS_LINK, $this->HIT_NUCLEUS_BLOCKOTHERSOUNDS, $this->HIT_NUCLEUS_DURATION, $shooter->login);
				break;
			// ARROW
			case '5':
				// Play sound ?
				$this->soundToPlay(self::HIT_ARROW_ENABLED_TO, $this->HIT_ARROW_LINK, $this->HIT_ARROW_BLOCKOTHERSOUNDS, $this->HIT_ARROW_DURATION, $shooter->login);
				break;
		}
		
		// Headshot ?
		//$this->DoHeadShot($weapon, $shooter, $victim);
	}
	
	
	/**
	 * Function called when a player has no more armor (killed by weapon, killed by offzone)
	 *
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/ArmorEmptyStructure.php
	public function onArmorEmpty(ArmorEmptyStructure $structure) {
		// Get informations
		$shooter = $structure->getShooter();
		if ($shooter != null) {
			$shooterLogin = $shooter->login;
			$shooterNickname = $shooter->nickname;
			$shooterTeamId = $shooter->teamId;
			$shooterScore = $this->playersScore[$shooterLogin];
		}
		$victim = $structure->getVictim();
		if ($victim != null) {
			$victimLogin = $victim->login;
			$victimNickname = $victim->nickname;
			$victimTeamId = $victim->teamId;
			$victimScore = $this->playersScore[$victimLogin];
		}
		$damage = $structure->getDamage();
		$shooterPoints = $structure->getShooterPoints();
		$weapon = $structure->getWeapon();
		
		// DEBUG
		//$msg = 'onArmorEmpty() : shooter=' . $structure->getShooter() . ' / victim=' . $structure->getVictim() . ' / damage=' . $structure->getDamage() . ' / shooterPoints=' . $structure->getShooterPoints() . ' / weapon=' . $structure->getWeapon();
		//if ($shooter != null && $victim != null) { $this->toLog($shooterNickname . ' killed ' . $victimNickname . ' !'); }
		
		// KILL BY PLAYER
		if ($shooter != null) {
			// TEAMKILLER (excepted for FFA modes : teamId = -1)
			//if ($shooterTeamId == $victimTeamId && $this->maniaControl->server->titleId != 'SMStormRoyal@nadeolabs') {
			if ($shooterTeamId == $victimTeamId && $shooterTeamId != -1) {
				$rc = $this->soundToPlay(self::TEAMKILLER_ENABLED_TO, $this->TEAMKILLER_LINK, $this->TEAMKILLER_BLOCKOTHERSOUNDS, $this->TEAMKILLER_DURATION, $shooterLogin);
				if ($rc) { $this->showMessage($shooterNickname . '$z$s$fff : Team killer !', self::TEAMKILLER_ENABLED_TO, $shooterLogin); }
			}
			// KILLSTREAK
			else {
				$this->playersScore[$shooterLogin] = $shooterScore + 1;
				
				// Sound to play ?
				$this->killAwards($shooter);
				
				// NB : Delai on killAwards() : KO, can't pass parameter in Timer Callback
			}
		}
		
		// DEATH
		if ($victim != null) {
			$this->playersScore[$victimLogin] = '0';
			
			// DEATH BY OFF-ZONE
			if ($shooter == null) {
				$rc = $this->soundToPlay(self::OFFZONE_ENABLED_TO, $this->OFFZONE_LINK, $this->OFFZONE_BLOCKOTHERSOUNDS, $this->OFFZONE_DURATION, $victimLogin);
				if ($rc) { $this->showMessage($victimNickname . '$z$s$fff : Off-zoned !', self::OFFZONE_ENABLED_TO, $victimLogin); }
			}
		}
		
		// Update RANKING
		$this->ranking();
	}
	
	
	
	/**
	 * Get the ranking of the 3 best killstreakers and display a widget of killstreak ranking
	 */
	public function ranking() {
		$killStreak_1_login	= null;
		$killStreak_1_score	= 0;
		$killStreak_2_login	= null;
		$killStreak_2_score	= 0;
		$killStreak_3_login	= null;
		$killStreak_3_score	= 0;
		
		// Sort array by descending scores
		arsort($this->playersScore);
		$i = 0;
		// Get the 3 best killstreak players with a score >= 1st score to play sounds
		foreach ($this->playersScore as $login => $score) {
			// Don't treat IGNORED players
			if ($this->isIgnored($login)){ break; }
			
			// Don't treat scores lower than the minimum score needed to play the 1st sound
			//$this->toLog('score=' . $score . ' for ' . $login . ' (min=' . $this->scoreMin . ' - max=' . $this->scoreMax . ')');
			if ($score < $this->scoreMin) { break; }
			
			$i = $i + 1;
			
			// 1st best killstreak player
			if ($i == 1) {
				$killStreak_1_login	= $login;
				$killStreak_1_score	= $score;
			}
			// 2nd best killstreak player
			else if ($i == 2) {
				$killStreak_2_login	= $login;
				$killStreak_2_score	= $score;
			}
			// 3rd best killstreak player
			else if ($i == 3) {
				$killStreak_3_login	= $login;
				$killStreak_3_score	= $score;
			}
			
			// Get only 3 best players
			if ($i == 3) { break; }
		}
		
		// Display the widget of killstreak ranking
		$this->showRanking($killStreak_1_login, $killStreak_1_score, $killStreak_2_login, $killStreak_2_score, $killStreak_3_login, $killStreak_3_score);
	}
	
	/**
	 * Play sounds if the $player has a good score
	 *
	 * @param Player player
	 */
	public function killAwards(Player $player) {
		$playerLogin = $player->login;
		$playerNickname = $player->nickname;
		
		// Kill awards activated ?
		$KILL_AWARDS_ENABLED_TO = $this->maniaControl->settingManager->getSettingValue($this, self::KILL_AWARDS_ENABLED_TO);
		if ($KILL_AWARDS_ENABLED_TO == 'none') { return; }
		
		/*
		// Get dynamicaly all KILL_*_LINK parameters and their value
		for ($i = 1; $i <= 30 ; $i++) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// Get parameters
			${"KILL_" . $i_2digits . "_LINK"} = $this->maniaControl->settingManager->getSettingValue($this, constant("KILL_" . $i_2digits . "_LINK"));
		}
		*/
		
		// Last playable KILL_LINK and KILLSTREAK associated ?
		// For all 30 variables, starting from the last...
		for ($i = 30; $i >= 0; $i--) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			$SOUND_LINK = $this->{"KILL_" . $i_2digits . "_LINK"};
			// if the "KILL_LINK" variable is not empty...
			if ($SOUND_LINK != '') {
				// Store KILL_LINK and KILLSTREAK associated
				$this->scoreMax = $i;
				$this->soundLinkMax = $SOUND_LINK;
				break;
			}
		}
		
		// First playable KILL_LINK and KILLSTREAK associated ?
		// For all 30 variables, starting from the first...
		for ($i = 1; $i <= 30 ; $i++) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			$SOUND_LINK = $this->{"KILL_" . $i_2digits . "_LINK"};
			// if the "KILL_LINK" variable is not empty...
			if ($SOUND_LINK != '') {
				// Store KILL_LINK and KILLSTREAK associated
				$this->scoreMin = $i;
				$this->soundLinkMin = $SOUND_LINK;
				break;
			}
		}
		
		// Play sound depending on player's score
		if (!is_null($playerLogin)) {
			$playerScore = $this->playersScore[$playerLogin];
			// Infinite "last-sound" if scoreMax hited
			if ($playerScore >= $this->scoreMax) { 
				$SOUND_BLOCKOTHERSOUNDS	= $this->{"KILL_30_BLOCKOTHERSOUNDS"};
				$SOUND_DURATION			= $this->{"KILL_30_DURATION"};
				$rc = $this->playSound($KILL_AWARDS_ENABLED_TO, $this->soundLinkMax, $SOUND_BLOCKOTHERSOUNDS, $SOUND_DURATION, $playerLogin, 'sound');
				if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !!', self::KILL_AWARDS_ENABLED_TO, $playerLogin); }
			}
			// Or just sound depending on killstreak of the player
			else {
				$playerScore_2digits = sprintf("%02u",$playerScore);
				$SOUND_LINK				= $this->{"KILL_" . $playerScore_2digits . "_LINK"};
				$SOUND_BLOCKOTHERSOUNDS	= $this->{"KILL_" . $playerScore_2digits . "_BLOCKOTHERSOUNDS"};
				$SOUND_DURATION			= $this->{"KILL_" . $playerScore_2digits . "_DURATION"};
				$rc = $this->playSound($KILL_AWARDS_ENABLED_TO, $SOUND_LINK, $SOUND_BLOCKOTHERSOUNDS, $SOUND_DURATION, $playerLogin, 'sound');
				if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !', self::KILL_AWARDS_ENABLED_TO, $playerLogin); }
			}
		}
	}
	
	
	
	// Cf SBXmlRpc::OnPass(_Event) from https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Modes/ShootMania/SpeedBall.Script.txt
	// which calls onPass() from https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Libs/steeffeen/ShootMania/SpeedBallXmlRpc.Script.txt
	public function onPass(Array $array) {
		$shooterLogin = $array[1][0];
		$victimLogin = $array[1][1];
		$shooterNickname = $this->maniaControl->getPlayerManager()->getPlayer($shooterLogin)->nickname;
		$victimNickname = $this->maniaControl->getPlayerManager()->getPlayer($victimLogin)->nickname;
		
		$this->toLog($shooterNickname . ' $z$s$fff : Nice pass to ' . $victimNickname . ' !');
	}
	
	
	
	/**
	 * Function called when a weapon (all ?) was close to an enemy
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/NearMissStructure.php
	// TO TEST
	/* // V1.08 
	public function onNearMiss(NearMissStructure $structure) {
		// Get informations
		$shooter = $structure->getShooter();
		if ($shooter != null) {
			$shooterLogin = $shooter->login;
			$shooterNickname = $shooter->nickname;
			$shooterTeamId = $shooter->teamId;
			$shooterScore = $this->playersScore[$shooterLogin];
		}
		$victim = $structure->getVictim();
		if ($victim != null) {
			$victimLogin = $victim->login;
			$victimNickname = $victim->nickname;
			$victimTeamId = $victim->teamId;
			$victimScore = $this->playersScore[$victimLogin];
		}
		$distance= $structure->getDistance();
		$weapon = $structure->getWeapon();
		
		// DEBUG
		$this->toLog($shooterNickname . '$z$s$fff missed ' . $victimNickname . '$z$s$fff from ' . $distance . ' cm with weapon ' . $weapon);
		
		// Play sound ?
		$this->soundToPlay(self::ONNEARMISS_ENABLED_TO, $this->ONNEARMISS_LINK, $this->ONNEARMISS_BLOCKOTHERSOUNDS, $this->ONNEARMISS_DURATION, $shooterLogin);
	}
	*/
	
	
	
	/**
	 * Function called when the Warm Up begins
	 */
	// TO TEST
	/* // V1.08 
	public function onBeginWarmup() {
		// DEBUG
		$this->toLog("callback onBeginWarmup()");
		
		// Play sound ?
		$this->soundToPlay(self::BEGINWARMUP_ENABLED, $this->BEGINWARMUP_LINK, $this->BEGINWARMUP_BLOCKOTHERSOUNDS, $this->BEGINWARMUP_DURATION, null);
	}
	*/
	
	
	
	/**
	 * Function called when the Warm Up ends
	 */
	 // TO TEST
	 /* // V1.08 
	public function onEndWarmup() {
		// DEBUG
		$this->toLog("callback onEndWarmup()");
		
		// Play sound ?
		$this->soundToPlay(self::ENDWARMUP_ENABLED, $this->ENDWARMUP_LINK, $this->ENDWARMUP_BLOCKOTHERSOUNDS, $this->ENDWARMUP_DURATION, null);
	}
	*/
	
	
	
	/**
	 * Function called when a round is over
	 */
	 // TO TEST
	 // NB : joué en même temps que "onCapture" (1s avant)
	 /* // V1.08 
	public function onEndPlaying() {
		// DEBUG
		$this->toLog("callback onEndPlaying()");
		
		// Play sound ?
		$this->soundToPlay(self::ENDPLAYING_ENABLED, $this->ENDPLAYING_LINK, $this->ENDPLAYING_BLOCKOTHERSOUNDS, $this->ENDPLAYING_DURATION, null);
	}
	*/
	
	
	
	/**
	 * Function called when a map ends
	 */
	 // TO TEST
	 /* // V1.08 
	public function onEndMap() {
		// DEBUG
		$this->toLog("callback onEndMap()");
		
		// Play sound ?
		$this->soundToPlay(self::ENDMAP_ENABLED, $this->ENDMAP_LINK, $this->ENDMAP_BLOCKOTHERSOUNDS, $this->ENDMAP_DURATION, null);
	}
	*/
	
	
	
	/*
	// KO, never trigerred
	public function onAfkStatus() {
		$this->toLog("callback onAfkStatus()");
	}
	// KO, never trigerred
	public function onBeginPodium() {
		$this->toLog("callback onBeginPodium()");
	}
	// KO, never trigerred
	public function onEndPodium() {
		$this->toLog("callback onEndPodium()");
	}
	public function onPlayerAlliesChanged() {
		$this->toLog("callback onPlayerAlliesChanged()");
	}
	public function onPlayerInfoChanged() {
		$this->toLog("callback onPlayerInfoChanged()");
	}
	*/
	
	
	
	/**
	 * Function called to display callbacks in DEBUG mode
	 */
	public function callbacksForDebug_string($callbackName) {
		//$this->toLog('callback (string) ' . $callbackName);
		
		//var_dump($callbackName); // DEBUG
	}
	
	/**
	 * Function called to display callbacks in DEBUG mode
	 */ // TO DO : String passé en paramètre lors d'un appel
	//public function callbacksForDebug(array $callback) {
	public function callbacksForDebug_array(array $callback) {
		$callbackName = $callback[1][0];
		
		$this->toLog('callback (array) ' . $callbackName); // FONCTIONNE COMME (string)
	}
	
	
	
	
	/*
	// TO DO : ajouter commentaires au début de la fonction
	*/
	public function ScanDirectory($ftp_server, $user, $password, $directory){
		$this->toLog("Scanning for files in $ftp_server");
		
		// Connect to the FTP server
		$conn_id = ftp_connect($ftp_server);
		if ($conn_id == false) {
			$this->maniaControl->getChat()->sendInformation("[Sound Events Manager] Error while connecting to $ftp_server", null, false);
			$this->sendInformation("Error while connecting to $ftp_server", null);
			return null;
		}

		// Identify to the FTP server
		$login_result = ftp_login($conn_id, $user, $password);
		if ($login_result == false) {
			//$this->maniaControl->getChat()->sendInformation("[Sound Events Manager] Error while logging with $user", null, false);
			$this->sendInformation("Error while logging with $user", null);
			return null;
		}
		
		// Switch to the target directory
		$chdir = ftp_chdir($conn_id, $directory);
		if ($chdir == false) {
			//$this->maniaControl->getChat()->sendInformation("[Sound Events Manager] Error while changing to directory $directory", null, false);
			$this->sendInformation("Error while changing to directory $directory", null);
			return null;
		}
		
		// Get all files recursively
		$filesList = $this->ftpRecursiveFileListing($conn_id, '.');
		
		// Disconnect to FTP
		ftp_close($conn_id);
		
		$this->toLog("Scan done.");
		
		return $filesList;
	}
	
	
	
	/** 
	 * Get a recursive listing of all files in all subfolders given an ftp handle and path
	 * 
	 * @param resource $ftpConnection  the ftp connection handle
	 * @param string $path  the folder/directory path
	 * @return array $allFiles the list of files
	 */
	public function ftpRecursiveFileListing($ftpConnection, $path) { 
		$allFiles = array();
		$contents = ftp_nlist($ftpConnection, $path);
		foreach($contents as $currentFile) {
			// Directory found : recursive call on this directory
			if (ftp_size($ftpConnection, $currentFile) == '-1') {
				$this->ftpRecursiveFileListing($ftpConnection, $currentFile); 
			}
			// File found
			else {
				// Add file to the list
				$allFiles[] = $currentFile;
				// DEBUG
				$this->toLog("File found : $currentFile");
			}
		}
		return $allFiles; 
	}
	
	
	
     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float
     */
    public static function getVersion() {
        return self::VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return self::DESCRIPTION;
    }
}


